@@ -70,14 +70,40 @@ namespace QuantLib {
                                    Frequency frequency,
                                    const Period& availabilityLag,
                                    Currency currency)
-    : familyName_(std::move(familyName)), region_(std::move(region)), revised_(revised),
-      interpolated_(interpolated), frequency_(frequency), availabilityLag_(availabilityLag),
-      currency_(std::move(currency)) {
-        name_ = region_.name() + " " + familyName_;
-        registerWith(Settings::instance().evaluationDate());
-        registerWith(IndexManager::instance().notifier(name()));
+    : InflationIndex(std::move(familyName),
+                     std::move(region),
+                     revised,
+                     frequency,
+                     availabilityLag,
+                     std::move(currency)) {
+        QL_DEPRECATED_DISABLE_WARNING
+        interpolated_ = interpolated;
+        QL_DEPRECATED_ENABLE_WARNING
     }
 
+    /* gcc complains in the constructor about the deprecated call of initializing
+     * `bool InflationIndex::interpolated_ = false;` that's why we need to disable the warning here.
+     */
+    QL_DEPRECATED_DISABLE_WARNING
+    InflationIndex::InflationIndex(std::string familyName,
+                                   Region region,
+                                   bool revised,
+                                   Frequency frequency,
+                                   const Period& availabilityLag,
+                                   Currency currency)
+    : familyName_(std::move(familyName)), region_(std::move(region)), revised_(revised),
+      frequency_(frequency), availabilityLag_(availabilityLag), currency_(std::move(currency)) {
+        name_ = region_.name() + " " + familyName_;
+        registerWith(Settings::instance().evaluationDate());
+        registerWith(IndexManager::instance().notifier(InflationIndex::name()));
+    }
+    QL_DEPRECATED_ENABLE_WARNING
+
+    bool InflationIndex::interpolated() const {
+        QL_DEPRECATED_DISABLE_WARNING
+        return interpolated_;
+        QL_DEPRECATED_ENABLE_WARNING
+    }
 
     Calendar InflationIndex::fixingCalendar() const {
         static NullCalendar c;
@@ -109,8 +135,21 @@ namespace QuantLib {
                                            const Period& availabilityLag,
                                            const Currency& currency,
                                            Handle<ZeroInflationTermStructure> zeroInflation)
-    : InflationIndex(
-          familyName, region, revised, interpolated, frequency, availabilityLag, currency),
+    : ZeroInflationIndex(
+          familyName, region, revised, frequency, availabilityLag, currency, std::move(zeroInflation)) {
+        QL_DEPRECATED_DISABLE_WARNING
+        interpolated_ = interpolated;
+        QL_DEPRECATED_ENABLE_WARNING
+    }
+
+    ZeroInflationIndex::ZeroInflationIndex(const std::string& familyName,
+                                           const Region& region,
+                                           bool revised,
+                                           Frequency frequency,
+                                           const Period& availabilityLag,
+                                           const Currency& currency,
+                                           Handle<ZeroInflationTermStructure> zeroInflation)
+    : InflationIndex(familyName, region, revised, frequency, availabilityLag, currency),
       zeroInflation_(std::move(zeroInflation)) {
         registerWith(zeroInflation_);
     }
@@ -125,7 +164,10 @@ namespace QuantLib {
             QL_REQUIRE(I1 != Null<Real>(),
                        "Missing " << name() << " fixing for " << p.first);
 
+            QL_DEPRECATED_DISABLE_WARNING
             if (interpolated_ && fixingDate > p.first) {
+            QL_DEPRECATED_ENABLE_WARNING
+
                 Real I2 = ts[p.second+1];
                 QL_REQUIRE(I2 != Null<Real>(),
                            "Missing " << name() << " fixing for " << p.second+1);
@@ -163,11 +205,13 @@ namespace QuantLib {
             inflationPeriod(todayMinusLag, frequency_).first-1;
         Date latestNeededDate = fixingDate;
 
+        QL_DEPRECATED_DISABLE_WARNING
         if (interpolated_) { // might need the next one too
             std::pair<Date,Date> p = inflationPeriod(fixingDate, frequency_);
             if (fixingDate > p.first)
                 latestNeededDate += Period(frequency_);
         }
+        QL_DEPRECATED_ENABLE_WARNING
 
         if (latestNeededDate <= historicalFixingKnown) {
             // the fixing date is well before the availability lag, so
@@ -181,7 +225,8 @@ namespace QuantLib {
             // we're not sure, but the fixing might be there so we
             // check.  Todo: check which fixings are not possible, to
             // avoid using fixings in the future
-            Real f = timeSeries()[latestNeededDate];
+            Date first = Date(1, latestNeededDate.month(), latestNeededDate.year());
+            Real f = timeSeries()[first];
             return (f == Null<Real>());
         }
     }
@@ -196,13 +241,15 @@ namespace QuantLib {
 
         std::pair<Date, Date> p = inflationPeriod(fixingDate, frequency_);
 
+        QL_DEPRECATED_DISABLE_WARNING
+
         Date firstDateInPeriod = p.first;
         Rate Z1 = zeroInflation_->zeroRate(firstDateInPeriod, Period(0,Days), false);
         Time t1 = inflationYearFraction(frequency_, interpolated_, zeroInflation_->dayCounter(),
                                         baseDate, firstDateInPeriod);
         Real I1 = baseFixing * std::pow(1.0 + Z1, t1);
 
-        if (interpolated() && fixingDate > firstDateInPeriod) {
+        if (interpolated_ && fixingDate > firstDateInPeriod) {
             Date firstDateInNextPeriod = p.second + 1;
             Rate Z2 = zeroInflation_->zeroRate(firstDateInNextPeriod, Period(0,Days), false);
             Time t2 = inflationYearFraction(frequency_, interpolated_, zeroInflation_->dayCounter(),
@@ -219,15 +266,21 @@ namespace QuantLib {
         } else {
             return I1;
         }
+
+        QL_DEPRECATED_ENABLE_WARNING
     }
 
 
     ext::shared_ptr<ZeroInflationIndex> ZeroInflationIndex::clone(
                           const Handle<ZeroInflationTermStructure>& h) const {
-        return ext::make_shared<ZeroInflationIndex>(
-                      familyName_, region_, revised_,
-                                             interpolated_, frequency_,
-                                             availabilityLag_, currency_, h);
+        /* using the new constructor and set interpolated to avoid the deprecated warning and
+         * error...  */
+        auto clonedIndex = ext::make_shared<ZeroInflationIndex>(
+            familyName_, region_, revised_, frequency_, availabilityLag_, currency_, h);
+        QL_DEPRECATED_DISABLE_WARNING
+        clonedIndex->interpolated_ = interpolated_;
+        QL_DEPRECATED_ENABLE_WARNING
+        return clonedIndex;
     }
 
     // these still need to be fixed to latest versions
@@ -241,9 +294,8 @@ namespace QuantLib {
                                          const Period& availabilityLag,
                                          const Currency& currency,
                                          Handle<YoYInflationTermStructure> yoyInflation)
-    : InflationIndex(
-          familyName, region, revised, interpolated, frequency, availabilityLag, currency),
-      ratio_(ratio), yoyInflation_(std::move(yoyInflation)) {
+    : InflationIndex(familyName, region, revised, frequency, availabilityLag, currency),
+      interpolated_(interpolated), ratio_(ratio), yoyInflation_(std::move(yoyInflation)) {
         registerWith(yoyInflation_);
     }
 
@@ -276,32 +328,26 @@ namespace QuantLib {
             if(interpolated()){ // IS ratio, IS interpolated
 
                 std::pair<Date,Date> lim = inflationPeriod(fixingDate, frequency_);
-                Date fixMinus1Y=NullCalendar().advance(fixingDate, -1*Years, ModifiedFollowing);
+                Date fixMinus1Y = NullCalendar().advance(fixingDate, -1*Years, ModifiedFollowing);
                 std::pair<Date,Date> limBef = inflationPeriod(fixMinus1Y, frequency_);
-                Real dp= lim.second + 1 - lim.first;
-                Real dpBef=limBef.second + 1 - limBef.first;
+                Real dp = lim.second + 1 - lim.first;
+                Real dpBef = limBef.second + 1 - limBef.first;
                 Real dl = fixingDate-lim.first;
                 // potentially does not work on 29th Feb
                 Real dlBef = fixMinus1Y - limBef.first;
                 // get the four relevant fixings
-                // recall that they are stored flat for every day
                 Rate limFirstFix = ts[lim.first];
                 QL_REQUIRE(limFirstFix != Null<Rate>(),
-                            "Missing " << name() << " fixing for "
-                            << lim.first );
+                           "Missing " << name() << " fixing for " << lim.first );
                 Rate limSecondFix = ts[lim.second+1];
                 QL_REQUIRE(limSecondFix != Null<Rate>(),
-                            "Missing " << name() << " fixing for "
-                            << lim.second+1 );
+                           "Missing " << name() << " fixing for " << lim.second+1 );
                 Rate limBefFirstFix = ts[limBef.first];
                 QL_REQUIRE(limBefFirstFix != Null<Rate>(),
-                            "Missing " << name() << " fixing for "
-                            << limBef.first );
-                Rate limBefSecondFix =
-                IndexManager::instance().getHistory(name())[limBef.second+1];
+                           "Missing " << name() << " fixing for " << limBef.first );
+                Rate limBefSecondFix = ts[limBef.second+1];
                 QL_REQUIRE(limBefSecondFix != Null<Rate>(),
-                            "Missing " << name() << " fixing for "
-                            << limBef.second+1 );
+                           "Missing " << name() << " fixing for " << limBef.second+1 );
 
                 Real linearNow = limFirstFix + (limSecondFix-limFirstFix)*dl/dp;
                 Real linearBef = limBefFirstFix + (limBefSecondFix-limBefFirstFix)*dlBef/dpBef;
@@ -310,15 +356,15 @@ namespace QuantLib {
                 return wasYES;
 
             } else {    // IS ratio, NOT interpolated
-                Rate pastFixing = ts[fixingDate];
+                std::pair<Date,Date> lim = inflationPeriod(fixingDate, frequency_);
+                Rate pastFixing = ts[lim.first];
                 QL_REQUIRE(pastFixing != Null<Rate>(),
-                            "Missing " << name() << " fixing for "
-                            << fixingDate);
+                            "Missing " << name() << " fixing for " << fixingDate);
                 Date previousDate = fixingDate - 1*Years;
-                Rate previousFixing = ts[previousDate];
+                std::pair<Date,Date> limBef = inflationPeriod(previousDate, frequency_);
+                Rate previousFixing = ts[limBef.first];
                 QL_REQUIRE(previousFixing != Null<Rate>(),
-                           "Missing " << name() << " fixing for "
-                           << previousDate );
+                           "Missing " << name() << " fixing for " << limBef.first );
 
                 return pastFixing/previousFixing - 1.0;
             }
@@ -328,8 +374,8 @@ namespace QuantLib {
             if (interpolated()) { // NOT ratio, IS interpolated
 
                 std::pair<Date,Date> lim = inflationPeriod(fixingDate, frequency_);
-                Real dp= lim.second + 1 - lim.first;
-                Real dl = fixingDate-lim.first;
+                Real dp = lim.second + 1 - lim.first;
+                Real dl = fixingDate - lim.first;
                 Rate limFirstFix = ts[lim.first];
                 QL_REQUIRE(limFirstFix != Null<Rate>(),
                             "Missing " << name() << " fixing for "
@@ -343,19 +389,16 @@ namespace QuantLib {
                 return linearNow;
 
             } else { // NOT ratio, NOT interpolated
-                    // so just flat
+                     // so just flat
 
-                Rate pastFixing = ts[fixingDate];
+                std::pair<Date,Date> lim = inflationPeriod(fixingDate, frequency_);
+                Rate pastFixing = ts[lim.first];
                 QL_REQUIRE(pastFixing != Null<Rate>(),
-                           "Missing " << name() << " fixing for "
-                           << fixingDate);
+                           "Missing " << name() << " fixing for " << lim.first);
                 return pastFixing;
 
             }
         }
-
-        // QL_FAIL("YoYInflationIndex::fixing, should never get here");
-
     }
 
 
@@ -386,9 +429,12 @@ namespace QuantLib {
     detail::CPI::effectiveInterpolationType(const ext::shared_ptr<ZeroInflationIndex>& index,
                                             const QuantLib::CPI::InterpolationType& type) {
         if (type == QuantLib::CPI::AsIndex) {
+            QL_DEPRECATED_DISABLE_WARNING
             return index->interpolated() ? QuantLib::CPI::Linear : QuantLib::CPI::Flat;
+            QL_DEPRECATED_ENABLE_WARNING
         } else {
             return type;
         }
     }
+
 }
