@@ -48,13 +48,6 @@ namespace QuantLib {
         3. Choose position type = Short for an "FRA sale" (future short
            loan, long deposit [lender])
 
-        4. If strike is given in the constructor, can calculate the NPV
-           of the contract via NPV().
-
-        5. If forward rate is desired/unknown, it can be obtained via
-           forwardRate(). In this case, the strike variable in the
-           constructor is irrelevant and will be ignored.
-
         <b>Example: </b>
         \link FRA.cpp
         valuation of a forward-rate agreement
@@ -62,10 +55,6 @@ namespace QuantLib {
 
         \todo Add preconditions and tests
 
-        \todo Should put an instance of ForwardRateAgreement in the
-              FraRateHelper to ensure consistency with the piecewise
-              yield curve.
-
         \todo Differentiate between BBA (British)/AFB (French)
               [assumed here] and ABA (Australian) banker conventions
               in the calculations.
@@ -74,31 +63,40 @@ namespace QuantLib {
 
         \ingroup instruments
     */
-    class ForwardRateAgreement: public Forward {
+    class ForwardRateAgreement: public Instrument {
       public:
-        ForwardRateAgreement(const Date& valueDate,
-                             const Date& maturityDate,
-                             Position::Type type,
-                             Rate strikeForwardRate,
-                             Real notionalAmount,
-                             const ext::shared_ptr<IborIndex>& index,
-                             const Handle<YieldTermStructure>& discountCurve =
-                                                 Handle<YieldTermStructure>(),
-                             bool useIndexedCoupon = true);
+        ForwardRateAgreement(
+            const Date& valueDate,
+            const Date& maturityDate,
+            Position::Type type,
+            Rate strikeForwardRate,
+            Real notionalAmount,
+            const ext::shared_ptr<IborIndex>& index,
+            Handle<YieldTermStructure> discountCurve = Handle<YieldTermStructure>(),
+            bool useIndexedCoupon = true);
+        ForwardRateAgreement(
+            const Date& valueDate,
+            Position::Type type,
+            Rate strikeForwardRate,
+            Real notionalAmount,
+            const ext::shared_ptr<IborIndex>& index,
+            Handle<YieldTermStructure> discountCurve = Handle<YieldTermStructure>());
+
         //! \name Calculations
         //@{
-        /*! A FRA expires/settles on the valueDate */
+        //! A FRA expires/settles on the value date
         bool isExpired() const override;
-        /*! This returns evaluationDate + settlementDays (not FRA
-            valueDate).
-        */
-        Date settlementDate() const override;
+        //! The payoff on the value date
+        Real amount() const;
+
+        const Calendar& calendar() const;
+        BusinessDayConvention businessDayConvention() const;
+        const DayCounter& dayCounter() const;
+        //! term structure relevant to the contract (e.g. repo curve)
+        Handle<YieldTermStructure> discountCurve() const;
+
         Date fixingDate() const;
-        /*!  Income is zero for a FRA */
-        Real spotIncome(const Handle<YieldTermStructure>& incomeDiscountCurve) const override;
-        //! Spot value (NPV) of the underlying loan
-        /*! This has always a positive value (asset), even if short the FRA */
-        Real spotValue() const override;
+
         //! Returns the relevant forward rate associated with the FRA term
         InterestRate forwardRate() const;
         //@}
@@ -115,12 +113,36 @@ namespace QuantLib {
         ext::shared_ptr<IborIndex> index_;
         bool useIndexedCoupon_;
 
+        DayCounter dayCounter_;
+        Calendar calendar_;
+        BusinessDayConvention businessDayConvention_;
+
+        //! the valueDate is the date the underlying index starts accruing and the FRA is settled.
+        Date valueDate_;
+        //! maturityDate of the underlying index; not the date the FRA is settled.
+        Date maturityDate_;
+        Handle<YieldTermStructure> discountCurve_;
+
       private:
         void calculateForwardRate() const;
+        void calculateAmount() const;
+        mutable Real amount_;
     };
 
+
+    inline const Calendar& ForwardRateAgreement::calendar() const { return calendar_; }
+
+    inline BusinessDayConvention ForwardRateAgreement::businessDayConvention() const {
+        return businessDayConvention_;
+    }
+
+    inline const DayCounter& ForwardRateAgreement::dayCounter() const { return dayCounter_; }
+
+    inline Handle<YieldTermStructure> ForwardRateAgreement::discountCurve() const {
+        return discountCurve_;
+    }
+
 }
 
 
 #endif
-
