@@ -27,6 +27,7 @@
 #include <ql/indexes/iborindex.hpp>
 #include <ql/instruments/vanillaswap.hpp>
 #include <ql/termstructures/yieldtermstructure.hpp>
+#include <ql/optional.hpp>
 #include <utility>
 
 namespace QuantLib {
@@ -40,7 +41,8 @@ namespace QuantLib {
                              ext::shared_ptr<IborIndex> iborIndex,
                              Spread spread,
                              DayCounter floatingDayCount,
-                             boost::optional<BusinessDayConvention> paymentConvention)
+                             ext::optional<BusinessDayConvention> paymentConvention,
+                             ext::optional<bool> useIndexedCoupons)
     : Swap(2), type_(type), nominal_(nominal), fixedSchedule_(std::move(fixedSchedule)),
       fixedRate_(fixedRate), fixedDayCount_(std::move(fixedDayCount)),
       floatingSchedule_(std::move(floatSchedule)), iborIndex_(std::move(iborIndex)),
@@ -60,7 +62,8 @@ namespace QuantLib {
             .withNotionals(nominal_)
             .withPaymentDayCounter(floatingDayCount_)
             .withPaymentAdjustment(paymentConvention_)
-            .withSpreads(spread_);
+            .withSpreads(spread_)
+            .withIndexedCoupons(useIndexedCoupons);
         for (Leg::const_iterator i = legs_[1].begin(); i < legs_[1].end(); ++i)
             registerWith(*i);
 
