@@ -175,45 +175,99 @@ namespace QuantLib {
     /*! \relates Array */
     Array operator+(const Array& v);
     /*! \relates Array */
+    Array operator+(Array&& v);
+    /*! \relates Array */
     Array operator-(const Array& v);
+    /*! \relates Array */
+    Array operator-(Array&& v);
 
     // binary operators
     /*! \relates Array */
     Array operator+(const Array&, const Array&);
     /*! \relates Array */
+    Array operator+(const Array&, Array&&);
+    /*! \relates Array */
+    Array operator+(Array&&, const Array&);
+    /*! \relates Array */
+    Array operator+(Array&&, Array&&);
+    /*! \relates Array */
     Array operator+(const Array&, Real);
     /*! \relates Array */
+    Array operator+(Array&&, Real);
+    /*! \relates Array */
     Array operator+(Real, const Array&);
     /*! \relates Array */
+    Array operator+(Real, Array&&);
+    /*! \relates Array */
     Array operator-(const Array&, const Array&);
     /*! \relates Array */
+    Array operator-(const Array&, Array&&);
+    /*! \relates Array */
+    Array operator-(Array&&, const Array&);
+    /*! \relates Array */
+    Array operator-(Array&&, Array&&);
+    /*! \relates Array */
     Array operator-(const Array&, Real);
     /*! \relates Array */
     Array operator-(Real, const Array&);
     /*! \relates Array */
+    Array operator-(Array&&, Real);
+    /*! \relates Array */
+    Array operator-(Real, Array&&);
+    /*! \relates Array */
     Array operator*(const Array&, const Array&);
     /*! \relates Array */
+    Array operator*(const Array&, Array&&);
+    /*! \relates Array */
+    Array operator*(Array&&, const Array&);
+    /*! \relates Array */
+    Array operator*(Array&&, Array&&);
+    /*! \relates Array */
     Array operator*(const Array&, Real);
     /*! \relates Array */
     Array operator*(Real, const Array&);
     /*! \relates Array */
+    Array operator*(Array&&, Real);
+    /*! \relates Array */
+    Array operator*(Real, Array&&);
+    /*! \relates Array */
     Array operator/(const Array&, const Array&);
     /*! \relates Array */
+    Array operator/(const Array&, Array&&);
+    /*! \relates Array */
+    Array operator/(Array&&, const Array&);
+    /*! \relates Array */
+    Array operator/(Array&&, Array&&);
+    /*! \relates Array */
     Array operator/(const Array&, Real);
     /*! \relates Array */
     Array operator/(Real, const Array&);
+    /*! \relates Array */
+    Array operator/(Array&&, Real);
+    /*! \relates Array */
+    Array operator/(Real, Array&&);
 
     // math functions
     /*! \relates Array */
     Array Abs(const Array&);
     /*! \relates Array */
+    Array Abs(Array&&);
+    /*! \relates Array */
     Array Sqrt(const Array&);
     /*! \relates Array */
+    Array Sqrt(Array&&);
+    /*! \relates Array */
     Array Log(const Array&);
     /*! \relates Array */
+    Array Log(Array&&);
+    /*! \relates Array */
     Array Exp(const Array&);
     /*! \relates Array */
+    Array Exp(Array&&);
+    /*! \relates Array */
     Array Pow(const Array&, Real);
+    /*! \relates Array */
+    Array Pow(Array&&, Real);
 
     // utilities
     /*! \relates Array */
@@ -242,10 +296,8 @@ namespace QuantLib {
 
     inline Array::Array(const Array& from)
     : data_(from.n_ != 0U ? new Real[from.n_] : (Real*)nullptr), n_(from.n_) {
-#if defined(QL_PATCH_MSVC) && defined(QL_DEBUG)
-        if (n_)
-        #endif
-        std::copy(from.begin(),from.end(),begin());
+        if (data_)
+            std::copy(from.begin(),from.end(),begin());
     }
 
     inline Array::Array(Array&& from) noexcept
@@ -327,8 +379,7 @@ namespace QuantLib {
         QL_REQUIRE(n_ == v.n_,
                    "arrays with different sizes (" << n_ << ", "
                    << v.n_ << ") cannot be added");
-        std::transform(begin(),end(),v.begin(),begin(),
-                       std::plus<Real>());
+        std::transform(begin(),end(),v.begin(),begin(), std::plus<>());
         return *this;
     }
 
@@ -342,8 +393,7 @@ namespace QuantLib {
         QL_REQUIRE(n_ == v.n_,
                    "arrays with different sizes (" << n_ << ", "
                    << v.n_ << ") cannot be subtracted");
-        std::transform(begin(),end(),v.begin(),begin(),
-                       std::minus<Real>());
+        std::transform(begin(), end(), v.begin(), begin(), std::minus<>());
         return *this;
     }
 
@@ -356,8 +406,7 @@ namespace QuantLib {
         QL_REQUIRE(n_ == v.n_,
                    "arrays with different sizes (" << n_ << ", "
                    << v.n_ << ") cannot be multiplied");
-        std::transform(begin(),end(),v.begin(),begin(),
-                       std::multiplies<Real>());
+        std::transform(begin(), end(), v.begin(), begin(), std::multiplies<>());
         return *this;
     }
 
@@ -370,8 +419,7 @@ namespace QuantLib {
         QL_REQUIRE(n_ == v.n_,
                    "arrays with different sizes (" << n_ << ", "
                    << v.n_ << ") cannot be divided");
-        std::transform(begin(),end(),v.begin(),begin(),
-                       std::divides<Real>());
+        std::transform(begin(), end(), v.begin(), begin(), std::divides<>());
         return *this;
     }
 
@@ -522,13 +570,21 @@ namespace QuantLib {
         return result;
     }
 
+    inline Array operator+(Array&& v) {
+        return std::move(v);
+    }
+
     inline Array operator-(const Array& v) {
         Array result(v.size());
-        std::transform(v.begin(),v.end(),result.begin(),
-                       std::negate<Real>());
+        std::transform(v.begin(), v.end(), result.begin(), std::negate<>());
         return result;
     }
 
+    inline Array operator-(Array&& v) {
+        Array result = std::move(v);
+        std::transform(result.begin(), result.end(), result.begin(), std::negate<>());
+        return result;
+    }
 
     // binary operators
 
@@ -537,8 +593,34 @@ namespace QuantLib {
                    "arrays with different sizes (" << v1.size() << ", "
                    << v2.size() << ") cannot be added");
         Array result(v1.size());
-        std::transform(v1.begin(),v1.end(),v2.begin(),result.begin(),
-                       std::plus<Real>());
+        std::transform(v1.begin(),v1.end(),v2.begin(),result.begin(), std::plus<>());
+        return result;
+    }
+
+    inline Array operator+(const Array& v1, Array&& v2) {
+        QL_REQUIRE(v1.size() == v2.size(),
+                   "arrays with different sizes (" << v1.size() << ", "
+                   << v2.size() << ") cannot be added");
+        Array result = std::move(v2);
+        std::transform(v1.begin(), v1.end(), result.begin(), result.begin(), std::plus<>());
+        return result;
+    }
+
+    inline Array operator+(Array&& v1, const Array& v2) {
+        QL_REQUIRE(v1.size() == v2.size(),
+                   "arrays with different sizes (" << v1.size() << ", "
+                   << v2.size() << ") cannot be added");
+        Array result = std::move(v1);
+        std::transform(result.begin(), result.end(), v2.begin(), result.begin(), std::plus<>());
+        return result;
+    }
+
+    inline Array operator+(Array&& v1, Array&& v2) {
+        QL_REQUIRE(v1.size() == v2.size(),
+                   "arrays with different sizes (" << v1.size() << ", "
+                   << v2.size() << ") cannot be added");
+        Array result = std::move(v2);
+        std::transform(v1.begin(), v1.end(), result.begin(), result.begin(), std::plus<>());
         return result;
     }
 
@@ -548,19 +630,57 @@ namespace QuantLib {
         return result;
     }
 
+    inline Array operator+(Array&& v1, Real a) {
+        Array result = std::move(v1);
+        std::transform(result.begin(), result.end(), result.begin(), [=](Real y) -> Real { return y + a; });
+        return result;
+    }
+
     inline Array operator+(Real a, const Array& v2) {
         Array result(v2.size());
         std::transform(v2.begin(),v2.end(),result.begin(), [=](Real y) -> Real { return a + y; });
         return result;
     }
 
+    inline Array operator+(Real a, Array&& v2) {
+        Array result = std::move(v2);
+        std::transform(result.begin(), result.end(), result.begin(), [=](Real y) -> Real { return a + y; });
+        return result;
+    }
+
     inline Array operator-(const Array& v1, const Array& v2) {
         QL_REQUIRE(v1.size() == v2.size(),
                    "arrays with different sizes (" << v1.size() << ", "
                    << v2.size() << ") cannot be subtracted");
         Array result(v1.size());
-        std::transform(v1.begin(),v1.end(),v2.begin(),result.begin(),
-                       std::minus<Real>());
+        std::transform(v1.begin(), v1.end(), v2.begin(), result.begin(), std::minus<>());
+        return result;
+    }
+
+    inline Array operator-(const Array& v1, Array&& v2) {
+        QL_REQUIRE(v1.size() == v2.size(),
+                   "arrays with different sizes (" << v1.size() << ", "
+                   << v2.size() << ") cannot be subtracted");
+        Array result = std::move(v2);
+        std::transform(v1.begin(), v1.end(), result.begin(), result.begin(), std::minus<>());
+        return result;
+    }
+
+    inline Array operator-(Array&& v1, const Array& v2) {
+        QL_REQUIRE(v1.size() == v2.size(),
+                   "arrays with different sizes (" << v1.size() << ", "
+                   << v2.size() << ") cannot be subtracted");
+        Array result = std::move(v1);
+        std::transform(result.begin(), result.end(), v2.begin(), result.begin(), std::minus<>());
+        return result;
+    }
+
+    inline Array operator-(Array&& v1, Array&& v2) {
+        QL_REQUIRE(v1.size() == v2.size(),
+                   "arrays with different sizes (" << v1.size() << ", "
+                   << v2.size() << ") cannot be subtracted");
+        Array result = std::move(v2);
+        std::transform(v1.begin(), v1.end(), result.begin(), result.begin(), std::minus<>());
         return result;
     }
 
@@ -570,19 +690,57 @@ namespace QuantLib {
         return result;
     }
 
+    inline Array operator-(Array&& v1, Real a) {
+        Array result = std::move(v1);
+        std::transform(result.begin(), result.end(), result.begin(), [=](Real y) -> Real { return y - a; });
+        return result;
+    }
+
     inline Array operator-(Real a, const Array& v2) {
         Array result(v2.size());
         std::transform(v2.begin(),v2.end(),result.begin(), [=](Real y) -> Real { return a - y; });
         return result;
     }
 
+    inline Array operator-(Real a, Array&& v2) {
+        Array result = std::move(v2);
+        std::transform(result.begin(), result.end(), result.begin(), [=](Real y) -> Real { return a - y; });
+        return result;
+    }
+
     inline Array operator*(const Array& v1, const Array& v2) {
         QL_REQUIRE(v1.size() == v2.size(),
                    "arrays with different sizes (" << v1.size() << ", "
                    << v2.size() << ") cannot be multiplied");
         Array result(v1.size());
-        std::transform(v1.begin(),v1.end(),v2.begin(),result.begin(),
-                       std::multiplies<Real>());
+        std::transform(v1.begin(), v1.end(), v2.begin(), result.begin(), std::multiplies<>());
+        return result;
+    }
+
+    inline Array operator*(const Array& v1, Array&& v2) {
+        QL_REQUIRE(v1.size() == v2.size(),
+                   "arrays with different sizes (" << v1.size() << ", "
+                   << v2.size() << ") cannot be multiplied");
+        Array result = std::move(v2);
+        std::transform(v1.begin(), v1.end(), result.begin(), result.begin(), std::multiplies<>());
+        return result;
+    }
+
+    inline Array operator*(Array&& v1, const Array& v2) {
+        QL_REQUIRE(v1.size() == v2.size(),
+                   "arrays with different sizes (" << v1.size() << ", "
+                   << v2.size() << ") cannot be multiplied");
+        Array result = std::move(v1);
+        std::transform(result.begin(), result.end(), v2.begin(), result.begin(), std::multiplies<>());
+        return result;
+    }
+
+    inline Array operator*(Array&& v1, Array&& v2) {
+        QL_REQUIRE(v1.size() == v2.size(),
+                   "arrays with different sizes (" << v1.size() << ", "
+                   << v2.size() << ") cannot be multiplied");
+        Array result = std::move(v2);
+        std::transform(v1.begin(), v1.end(), result.begin(), result.begin(), std::multiplies<>());
         return result;
     }
 
@@ -592,19 +750,57 @@ namespace QuantLib {
         return result;
     }
 
+    inline Array operator*(Array&& v1, Real a) {
+        Array result = std::move(v1);
+        std::transform(result.begin(), result.end(), result.begin(), [=](Real y) -> Real { return y * a; });
+        return result;
+    }
+
     inline Array operator*(Real a, const Array& v2) {
         Array result(v2.size());
         std::transform(v2.begin(),v2.end(),result.begin(), [=](Real y) -> Real { return a * y; });
         return result;
     }
 
+    inline Array operator*(Real a, Array&& v2) {
+        Array result = std::move(v2);
+        std::transform(result.begin(), result.end(), result.begin(), [=](Real y) -> Real { return a * y; });
+        return result;
+    }
+
     inline Array operator/(const Array& v1, const Array& v2) {
         QL_REQUIRE(v1.size() == v2.size(),
                    "arrays with different sizes (" << v1.size() << ", "
                    << v2.size() << ") cannot be divided");
         Array result(v1.size());
-        std::transform(v1.begin(),v1.end(),v2.begin(),result.begin(),
-                       std::divides<Real>());
+        std::transform(v1.begin(), v1.end(), v2.begin(), result.begin(), std::divides<>());
+        return result;
+    }
+
+    inline Array operator/(const Array& v1, Array&& v2) {
+        QL_REQUIRE(v1.size() == v2.size(),
+                   "arrays with different sizes (" << v1.size() << ", "
+                   << v2.size() << ") cannot be divided");
+        Array result = std::move(v2);
+        std::transform(v1.begin(), v1.end(), result.begin(), result.begin(), std::divides<>());
+        return result;
+    }
+
+    inline Array operator/(Array&& v1, const Array& v2) {
+        QL_REQUIRE(v1.size() == v2.size(),
+                   "arrays with different sizes (" << v1.size() << ", "
+                   << v2.size() << ") cannot be divided");
+        Array result = std::move(v1);
+        std::transform(result.begin(), result.end(), v2.begin(), result.begin(), std::divides<>());
+        return result;
+    }
+
+    inline Array operator/(Array&& v1, Array&& v2) {
+        QL_REQUIRE(v1.size() == v2.size(),
+                   "arrays with different sizes (" << v1.size() << ", "
+                   << v2.size() << ") cannot be divided");
+        Array result = std::move(v2);
+        std::transform(v1.begin(), v1.end(), result.begin(), result.begin(), std::divides<>());
         return result;
     }
 
@@ -614,12 +810,24 @@ namespace QuantLib {
         return result;
     }
 
+    inline Array operator/(Array&& v1, Real a) {
+        Array result = std::move(v1);
+        std::transform(result.begin(), result.end(), result.begin(), [=](Real y) -> Real { return y / a; });
+        return result;
+    }
+
     inline Array operator/(Real a, const Array& v2) {
         Array result(v2.size());
         std::transform(v2.begin(),v2.end(),result.begin(), [=](Real y) -> Real { return a / y; });
         return result;
     }
 
+    inline Array operator/(Real a, Array&& v2) {
+        Array result = std::move(v2);
+        std::transform(result.begin(), result.end(), result.begin(), [=](Real y) -> Real { return a / y; });
+        return result;
+    }
+
     // functions
 
     inline Array Abs(const Array& v) {
@@ -629,6 +837,13 @@ namespace QuantLib {
         return result;
     }
 
+    inline Array Abs(Array&& v) {
+        Array result = std::move(v);
+        std::transform(result.begin(), result.end(), result.begin(),
+                       [](Real x) -> Real { return std::fabs(x); });
+        return result;
+    }
+
     inline Array Sqrt(const Array& v) {
         Array result(v.size());
         std::transform(v.begin(),v.end(),result.begin(),
@@ -636,6 +851,13 @@ namespace QuantLib {
         return result;
     }
 
+    inline Array Sqrt(Array&& v) {
+        Array result = std::move(v);
+        std::transform(result.begin(), result.end(), result.begin(),
+                       [](Real x) -> Real { return std::sqrt(x); });
+        return result;
+    }
+
     inline Array Log(const Array& v) {
         Array result(v.size());
         std::transform(v.begin(),v.end(),result.begin(),
@@ -643,6 +865,13 @@ namespace QuantLib {
         return result;
     }
 
+    inline Array Log(Array&& v) {
+        Array result = std::move(v);
+        std::transform(result.begin(), result.end(), result.begin(),
+                       [](Real x) -> Real { return std::log(x); });
+        return result;
+    }
+
     inline Array Exp(const Array& v) {
         Array result(v.size());
         std::transform(v.begin(), v.end(), result.begin(),
@@ -650,13 +879,26 @@ namespace QuantLib {
         return result;
     }
 
-    inline Array Pow(const Array& v, Real alpha) {
-        Array result(v.size());
-        for (Size i=0; i<v.size(); ++i)
-            result[i] = std::pow(v[i], alpha);
+    inline Array Exp(Array&& v) {
+        Array result = std::move(v);
+        std::transform(result.begin(), result.end(), result.begin(),
+                       [](Real x) -> Real { return std::exp(x); });
         return result;
     }
 
+    inline Array Pow(const Array& v, Real alpha) {
+        Array result(v.size());
+        std::transform(v.begin(), v.end(), result.begin(),
+                       [=](Real x) -> Real { return std::pow(x, alpha); });
+        return result;
+    }
+
+    inline Array Pow(Array&& v, Real alpha) {
+        Array result = std::move(v);
+        std::transform(result.begin(), result.end(), result.begin(),
+                       [=](Real x) -> Real { return std::pow(x, alpha); });
+        return result;
+    }
 
     inline void swap(Array& v, Array& w) {
         v.swap(w);
