@@ -37,8 +37,10 @@
 #include <ql/time/calendars/unitedstates.hpp>
 #include <ql/time/calendars/weekendsonly.hpp>
 #include <ql/currencies/america.hpp>
+#include <ql/currencies/europe.hpp>
 #include <ql/time/daycounters/actual360.hpp>
 #include <ql/time/daycounters/thirty360.hpp>
+#include <ql/optional.hpp>
 #include <map>
 #include <iomanip>
 #include <iostream>
@@ -647,26 +649,29 @@ void CreditDefaultSwapTest::testIsdaEngine() {
     Rate spreads[] = {0.001, 0.1};
     Rate recoveries[] = {0.2, 0.4};
 
-    double markitValues[] = {97798.29358, //0.001
-                             97776.11889, //0.001
-                             -914971.5977, //0.1
-                             -894985.6298, //0.1
-                             186921.3594, //0.001
-                             186839.8148, //0.001
-                             -1646623.672, //0.1
-                             -1579803.626, //0.1
-                             274298.9203,
-                             274122.4725,
-                             -2279730.93,
-                             -2147972.527,
-                             592420.2297,
-                             591571.2294,
-                             -3993550.206,
-                             -3545843.418,
-                             797501.1422,
-                             795915.9787,
-                             -4702034.688,
-                             -4042340.999};
+    double markitValues[] = {
+         -97798.29358, //0.001
+         -97776.11889, //0.001
+         914971.5977,  //0.1
+         894985.6298,  //0.1
+        -186921.3594,  //0.001
+        -186839.8148,  //0.001
+        1646623.672,   //0.1
+        1579803.626,   //0.1
+        -274298.9203,
+        -274122.4725,
+        2279730.93,
+        2147972.527,
+        -592420.2297,
+        -591571.2294,
+        3993550.206,
+        3545843.418,
+        -797501.1422,
+        -795915.9787,
+        4702034.688,
+        4042340.999
+    };
+
     /* When using indexes coupons, the risk-free curve is a bit off.
        We might skip the tests altogether and rely on running them
        with indexed coupons disabled, but leaving them can be useful anyway. */
@@ -676,19 +681,19 @@ void CreditDefaultSwapTest::testIsdaEngine() {
 
     for (auto termDate : termDates) {
         for (Real spread : spreads) {
-            for (Real& recoverie : recoveries) {
+            for (Real& recovery : recoveries) {
 
                 ext::shared_ptr<CreditDefaultSwap> quotedTrade =
                     MakeCreditDefaultSwap(termDate, spread).withNominal(10000000.);
 
                 Rate h = quotedTrade->impliedHazardRate(0., discountCurve, Actual365Fixed(),
-                                                        recoverie, 1e-10, CreditDefaultSwap::ISDA);
+                                                        recovery, 1e-10, CreditDefaultSwap::ISDA);
 
                 probabilityCurve.linkTo(
                     ext::make_shared<FlatHazardRate>(0, WeekendsOnly(), h, Actual365Fixed()));
 
                 ext::shared_ptr<IsdaCdsEngine> engine = ext::make_shared<IsdaCdsEngine>(
-                    probabilityCurve, recoverie, discountCurve, boost::none, IsdaCdsEngine::Taylor,
+                    probabilityCurve, recovery, discountCurve, ext::nullopt, IsdaCdsEngine::Taylor,
                     IsdaCdsEngine::HalfDayBias, IsdaCdsEngine::Piecewise);
 
                 ext::shared_ptr<CreditDefaultSwap> conventionalTrade =
@@ -697,7 +702,27 @@ void CreditDefaultSwapTest::testIsdaEngine() {
                         .withPricingEngine(engine);
 
                 QL_CHECK_CLOSE(conventionalTrade->notional() * conventionalTrade->fairUpfront(),
-                                  markitValues[l], tolerance);
+                               markitValues[l], tolerance);
+
+                // Now testing that with the calculated fair-upfront, both Buyer and Seller sides
+                // price close to zero
+                ext::shared_ptr<CreditDefaultSwap> conventionalTradeBuy =
+                    MakeCreditDefaultSwap(termDate, 0.01)
+                        .withNominal(10000000.)
+                        .withUpfrontRate(conventionalTrade->fairUpfront())
+                        .withSide(Protection::Buyer)
+                        .withPricingEngine(engine);
+
+                QL_CHECK_SMALL(conventionalTradeBuy->NPV(), tolerance);
+
+                ext::shared_ptr<CreditDefaultSwap> conventionalTradeSell =
+                    MakeCreditDefaultSwap(termDate, 0.01)
+                        .withNominal(10000000.)
+                        .withUpfrontRate(conventionalTrade->fairUpfront())
+                        .withSide(Protection::Seller)
+                        .withPricingEngine(engine);
+
+                QL_CHECK_SMALL(conventionalTradeSell->NPV(), tolerance);
 
                 l++;
             }
@@ -742,16 +767,223 @@ void CreditDefaultSwapTest::testAccrualRebateAmounts() {
     }
 }
 
+void CreditDefaultSwapTest::testIsdaCalculatorReconcileSingleQuote ()
+{
+    BOOST_TEST_MESSAGE(
+        "Testing ISDA engine calculations for a single credit-default swap record (reconciliation)...");
+
+    SavedSettings backup;
+
+    Date tradeDate(26, July, 2021);
+    Settings::instance().evaluationDate() = tradeDate;
+
+    //build an ISDA compliant yield curve
+    //data comes from Markit published rates
+    std::vector<ext::shared_ptr<RateHelper> > isdaRateHelpers;
+    int dep_tenors[] = {1, 3, 6, 12};
+    double dep_quotes[] = {-0.0056,-0.005440,-0.005190,-0.004930};
+
+    for(size_t i = 0; i < sizeof(dep_tenors) / sizeof(int); i++) {
+        isdaRateHelpers.push_back(ext::make_shared<DepositRateHelper>(
+                                     dep_quotes[i], dep_tenors[i] * Months, 2,
+                                     WeekendsOnly(), ModifiedFollowing,
+                                     false, Actual360()
+                                     )
+            );
+    }
+    int swap_tenors[] = {2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 15, 20, 30};
+    double swap_quotes[] = {-0.004820,
+                            -0.004420,
+                            -0.003990,
+                            -0.003520,
+                            -0.002970,
+                            -0.002370,
+                            -0.001760,
+                            -0.001140,
+                            -0.000540,
+                            0.000570,
+                            0.001880,
+                            0.002940,
+                            0.002820};
+
+    ext::shared_ptr<IborIndex> isda_ibor = ext::make_shared<IborIndex>(
+        "IsdaIbor", 6 * Months, 2, EURCurrency(), WeekendsOnly(),
+        ModifiedFollowing, false, Actual360());
+    for(size_t i = 0; i < sizeof(swap_tenors) / sizeof(int); i++) {
+        isdaRateHelpers.push_back(ext::make_shared<SwapRateHelper>(
+                                      swap_quotes[i], swap_tenors[i] * Years,
+                                      WeekendsOnly(),
+                                      Annual,
+                                      ModifiedFollowing, Thirty360(Thirty360::BondBasis), isda_ibor
+                                      )
+            );
+    }
+
+    RelinkableHandle<YieldTermStructure> discountCurve;
+    discountCurve.linkTo(
+            ext::make_shared<PiecewiseYieldCurve<Discount, LogLinear> >(
+                0, WeekendsOnly(), isdaRateHelpers, Actual365Fixed())
+        );
+
+    RelinkableHandle<DefaultProbabilityTermStructure> probabilityCurve;
+    Date instrumentMaturity = Date(20, June, 2026);
+    Rate coupon = 0.01, conventionalSpread = 0.006713, recovery = 0.4;
+    double nominal = 1e6, markitValue = -16070.7, expected_accrual = 1000, tolerance = 1.0e-3;
+
+    ext::shared_ptr<CreditDefaultSwap> quotedTrade =
+        MakeCreditDefaultSwap(instrumentMaturity, conventionalSpread).withNominal(nominal);
+
+    Rate h = quotedTrade->impliedHazardRate(0., discountCurve, Actual365Fixed(),
+                                            recovery, 1e-10, CreditDefaultSwap::ISDA);
+
+    probabilityCurve.linkTo(
+        ext::make_shared<FlatHazardRate>(0, WeekendsOnly(), h, Actual365Fixed()));
+
+    ext::shared_ptr<IsdaCdsEngine> engine = ext::make_shared<IsdaCdsEngine>(
+        probabilityCurve, recovery, discountCurve, ext::nullopt, IsdaCdsEngine::Taylor,
+        IsdaCdsEngine::HalfDayBias, IsdaCdsEngine::Piecewise);
+
+    ext::shared_ptr<CreditDefaultSwap> conventionalTrade =
+        MakeCreditDefaultSwap(instrumentMaturity, coupon)
+            .withNominal(nominal)
+            .withPricingEngine(engine);
+
+
+    Real npv = conventionalTrade->NPV();
+    Real calculated_upfront = conventionalTrade->notional() * conventionalTrade->fairUpfront();
+    Real df = calculated_upfront / npv; // to take into account of the discount to cash settlement
+    Real derived_accrual =
+        df * (npv -
+                                   conventionalTrade->defaultLegNPV() -
+                                   conventionalTrade->couponLegNPV());
+
+    Real calculated_accrual = conventionalTrade->accrualRebate()->amount();
+
+    auto settlement_date = conventionalTrade->accrualRebate()->date();
+
+    QL_CHECK_CLOSE(npv, markitValue, tolerance);
+
+    QL_CHECK_CLOSE(calculated_upfront, df * markitValue, tolerance);
+
+    QL_CHECK_CLOSE(derived_accrual, expected_accrual, tolerance);
+
+    QL_CHECK_CLOSE(calculated_accrual, expected_accrual, tolerance);
+
+    BOOST_CHECK_EQUAL(settlement_date, WeekendsOnly().advance(tradeDate,3, TimeUnit::Days));
+
+}
+
+void CreditDefaultSwapTest::testIsdaCalculatorReconcileSingleWithIssueDateInThePast ()
+{
+    BOOST_TEST_MESSAGE(
+        "Testing ISDA engine calculations for a single credit-default swap with issue date in the past...");
+
+    SavedSettings backup;
+
+    Date valueDate(26, July, 2021);
+    Settings::instance().evaluationDate() = valueDate;
+
+    //this is not IMM date but the settlement date is in the past so the accrual rebate
+    //should not be part of the NPV
+    Date tradeDate(20, July, 2019);
+
+    //build an ISDA compliant yield curve
+    //data comes from Markit published rates
+    std::vector<ext::shared_ptr<RateHelper> > isdaRateHelpers;
+    int dep_tenors[] = {1, 3, 6, 12};
+    double dep_quotes[] = {-0.0056,-0.005440,-0.005190,-0.004930};
+
+    for(size_t i = 0; i < sizeof(dep_tenors) / sizeof(int); i++) {
+        isdaRateHelpers.push_back(ext::make_shared<DepositRateHelper>(
+                                     dep_quotes[i], dep_tenors[i] * Months, 2,
+                                     WeekendsOnly(), ModifiedFollowing,
+                                     false, Actual360()
+                                     )
+            );
+    }
+    int swap_tenors[] = {2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 15, 20, 30};
+    double swap_quotes[] = {-0.004820,
+                            -0.004420,
+                            -0.003990,
+                            -0.003520,
+                            -0.002970,
+                            -0.002370,
+                            -0.001760,
+                            -0.001140,
+                            -0.000540,
+                            0.000570,
+                            0.001880,
+                            0.002940,
+                            0.002820};
+
+    ext::shared_ptr<IborIndex> isda_ibor = ext::make_shared<IborIndex>(
+        "IsdaIbor", 6 * Months, 2, EURCurrency(), WeekendsOnly(),
+        ModifiedFollowing, false, Actual360());
+    for(size_t i = 0; i < sizeof(swap_tenors) / sizeof(int); i++) {
+        isdaRateHelpers.push_back(ext::make_shared<SwapRateHelper>(
+                                      swap_quotes[i], swap_tenors[i] * Years,
+                                      WeekendsOnly(),
+                                      Annual,
+                                      ModifiedFollowing, Thirty360(Thirty360::BondBasis), isda_ibor
+                                      )
+            );
+    }
+
+    RelinkableHandle<YieldTermStructure> discountCurve;
+    discountCurve.linkTo(
+            ext::make_shared<PiecewiseYieldCurve<Discount, LogLinear> >(
+                0, WeekendsOnly(), isdaRateHelpers, Actual365Fixed())
+        );
+
+    RelinkableHandle<DefaultProbabilityTermStructure> probabilityCurve;
+    Date instrumentMaturity = Date(20, June, 2026);
+    Rate coupon = 0.01, conventionalSpread = 0.006713, recovery = 0.4;
+
+    //because there is no accrual involved, the markit value is decreased as compared to the
+    //previous test (old_markit_value - old_accrual or -16070.7 - 1000)
+    double nominal = 1e6, markitValue = -17070.77, expected_accrual = 0, tolerance = 1.0e-3;
+
+    ext::shared_ptr<CreditDefaultSwap> quotedTrade =
+        MakeCreditDefaultSwap(instrumentMaturity, conventionalSpread)
+        .withNominal(nominal);
+
+    Rate h = quotedTrade->impliedHazardRate(0., discountCurve, Actual365Fixed(),
+                                            recovery, 1e-10, CreditDefaultSwap::ISDA);
+
+    probabilityCurve.linkTo(
+        ext::make_shared<FlatHazardRate>(0, WeekendsOnly(), h, Actual365Fixed()));
+
+    ext::shared_ptr<IsdaCdsEngine> engine = ext::make_shared<IsdaCdsEngine>(
+        probabilityCurve, recovery, discountCurve, ext::nullopt, IsdaCdsEngine::Taylor,
+        IsdaCdsEngine::HalfDayBias, IsdaCdsEngine::Piecewise);
+
+    ext::shared_ptr<CreditDefaultSwap> conventionalTrade =
+        MakeCreditDefaultSwap(instrumentMaturity, coupon)
+            .withNominal(nominal)
+            .withPricingEngine(engine)
+            .withTradeDate(tradeDate);
+
+
+    Real npv = conventionalTrade->NPV();
+    Real calculated_accrual = npv -
+                                conventionalTrade->defaultLegNPV() -
+                                conventionalTrade->couponLegNPV();
+
+    QL_CHECK_CLOSE(npv, markitValue, tolerance);
+
+    QL_CHECK_CLOSE(calculated_accrual, expected_accrual, tolerance);
+}
+
 test_suite* CreditDefaultSwapTest::suite() {
     auto* suite = BOOST_TEST_SUITE("Credit-default swap tests");
     suite->add(QUANTLIB_TEST_CASE(&CreditDefaultSwapTest::testCachedValue));
-    suite->add(QUANTLIB_TEST_CASE(
-                              &CreditDefaultSwapTest::testCachedMarketValue));
-    suite->add(QUANTLIB_TEST_CASE(
-                              &CreditDefaultSwapTest::testImpliedHazardRate));
+    suite->add(QUANTLIB_TEST_CASE(&CreditDefaultSwapTest::testCachedMarketValue));
+    suite->add(QUANTLIB_TEST_CASE(&CreditDefaultSwapTest::testImpliedHazardRate));
     suite->add(QUANTLIB_TEST_CASE(&CreditDefaultSwapTest::testFairSpread));
     suite->add(QUANTLIB_TEST_CASE(&CreditDefaultSwapTest::testFairUpfront));
     suite->add(QUANTLIB_TEST_CASE(&CreditDefaultSwapTest::testIsdaEngine));
     suite->add(QUANTLIB_TEST_CASE(&CreditDefaultSwapTest::testAccrualRebateAmounts));
+    suite->add(QUANTLIB_TEST_CASE(&CreditDefaultSwapTest::testIsdaCalculatorReconcileSingleQuote));
+    suite->add(QUANTLIB_TEST_CASE(&CreditDefaultSwapTest::testIsdaCalculatorReconcileSingleWithIssueDateInThePast));
     return suite;
 }
