@@ -30,12 +30,14 @@ public:
     static void testPeriod();
     static void testZeroIndex();
     static void testZeroTermStructure();
+    static void testSeasonalityCorrection();
     static void testZeroIndexFutureFixing();
     static void testInterpolatedZeroTermStructure();
     static void testYYIndex();
     static void testYYTermStructure();
     static void testCpiFlatInterpolation();
-    static void testCpiInterpolation();
+    static void testCpiLinearInterpolation();
+    static void testCpiAsIndexInterpolation();
     static boost::unit_test_framework::test_suite* suite();
 };
 
