@@ -39,10 +39,6 @@
 #include <ql/indexes/inflation/euhicp.hpp>
 #include <ql/indexes/inflation/ukrpi.hpp>
 
-#ifndef LENGTH
-#define LENGTH(a) (sizeof(a)/sizeof(a[0]))
-#endif
-
 #include <iostream>
 
 
@@ -50,7 +46,6 @@
 //*************************************************************************
 namespace inflation_volatility_test {
 
-    using namespace std;
     using namespace QuantLib;
 
     // local data globals
@@ -60,18 +55,18 @@ namespace inflation_volatility_test {
     RelinkableHandle<YoYInflationTermStructure> yoyEU;
     RelinkableHandle<YoYInflationTermStructure> yoyUK;
 
-    vector<Rate> cStrikesEU;
-    vector<Rate> fStrikesEU;
-    vector<Period> cfMaturitiesEU;
+    std::vector<Rate> cStrikesEU;
+    std::vector<Rate> fStrikesEU;
+    std::vector<Period> cfMaturitiesEU;
     ext::shared_ptr<Matrix> cPriceEU;
     ext::shared_ptr<Matrix> fPriceEU;
 
     ext::shared_ptr<YoYInflationIndex> yoyIndexUK;
     ext::shared_ptr<YoYInflationIndex> yoyIndexEU;
 
-    vector<Rate> cStrikesUK;
-    vector<Rate> fStrikesUK;
-    vector<Period> cfMaturitiesUK;
+    std::vector<Rate> cStrikesUK;
+    std::vector<Rate> fStrikesUK;
+    std::vector<Period> cfMaturitiesUK;
     ext::shared_ptr<Matrix> cPriceUK;
     ext::shared_ptr<Matrix> fPriceUK;
 
@@ -133,8 +128,8 @@ namespace inflation_volatility_test {
                0.0498998, 0.0490464, 0.04768, 0.0464862, 0.045452,
                0.0437699, 0.0425311, 0.0420073, 0.041151};
 
-        vector <Real> r;
-        vector <Date> d;
+        std::vector <Real> r;
+        std::vector <Date> d;
         Size nTimesEUR = LENGTH(timesEUR);
         Size nTimesGBP = LENGTH(timesGBP);
         for (Size i = 0; i < nTimesEUR; i++) {
@@ -195,7 +190,7 @@ namespace inflation_volatility_test {
         ext::shared_ptr<InterpolatedYoYInflationCurve<Linear> >
             pYTSEU( new InterpolatedYoYInflationCurve<Linear>(
                     eval, TARGET(), Actual365Fixed(), Period(2,Months), Monthly,
-                    indexIsInterpolated, nominalGBP, d, r) );
+                    indexIsInterpolated, d, r) );
         yoyEU.linkTo(pYTSEU);
 
         // price data
@@ -226,9 +221,12 @@ namespace inflation_volatility_test {
         cStrikesEU.clear();
         fStrikesEU.clear();
         cfMaturitiesEU.clear();
-        for(Size i = 0; i < ncStrikesEU; i++) cStrikesEU.push_back(capStrikesEU[i]);
-        for(Size i = 0; i < nfStrikesEU; i++) fStrikesEU.push_back(floorStrikesEU[i]);
-        for(Size i = 0; i < ncfMaturitiesEU; i++) cfMaturitiesEU.push_back(capMaturitiesEU[i]);
+        for (Real& i : capStrikesEU)
+            cStrikesEU.push_back(i);
+        for (Real& i : floorStrikesEU)
+            fStrikesEU.push_back(i);
+        for (auto& i : capMaturitiesEU)
+            cfMaturitiesEU.push_back(i);
         ext::shared_ptr<Matrix> tcPriceEU(new Matrix(ncStrikesEU, ncfMaturitiesEU));
         ext::shared_ptr<Matrix> tfPriceEU(new Matrix(nfStrikesEU, ncfMaturitiesEU));
         for(Size i = 0; i < ncStrikesEU; i++) {
@@ -260,8 +258,7 @@ namespace inflation_volatility_test {
         DayCounter dc = Actual365Fixed();
         TARGET cal;
         BusinessDayConvention bdc = ModifiedFollowing;
-        ext::shared_ptr<QuantLib::YieldTermStructure> pn =
-            nominalEUR.currentLink();
+        const ext::shared_ptr<QuantLib::YieldTermStructure>& pn = nominalEUR.currentLink();
         Handle<QuantLib::YieldTermStructure> n(pn,false);
         ext::shared_ptr<InterpolatedYoYCapFloorTermPriceSurface<Bicubic,Cubic> >
         cfEUprices(new InterpolatedYoYCapFloorTermPriceSurface<Bicubic,Cubic>(
@@ -346,8 +343,7 @@ void InflationVolTest::testYoYPriceSurfaceToVol() {
     };
 
     Date d = yoySurf->baseDate() + Period(1,Years);
-    pair<vector<Rate>, vector<Volatility> > someSlice;
-    someSlice = yoySurf->Dslice(d);
+    auto someSlice = yoySurf->Dslice(d);
 
     Size n = someSlice.first.size();
     Real eps = 0.0001;
@@ -358,8 +354,7 @@ void InflationVolTest::testYoYPriceSurfaceToVol() {
     }
 
     d = yoySurf->baseDate() + Period(3,Years);
-    pair<vector<Rate>, vector<Volatility> >
-        someOtherSlice = yoySurf->Dslice(d);
+    auto someOtherSlice = yoySurf->Dslice(d);
     n = someOtherSlice.first.size();
     for(Size i = 0; i < n; i++){
         QL_REQUIRE(fabs(someOtherSlice.second[i]-volATyear3[i]) < eps,
@@ -385,8 +380,8 @@ void InflationVolTest::testYoYPriceSurfaceToATM() {
 
     setupPriceSurface();
 
-    pair<vector<Time>, vector<Rate> > yyATMt = priceSurfEU->atmYoYSwapTimeRates();
-    pair<vector<Date>, vector<Rate> > yyATMd = priceSurfEU->atmYoYSwapDateRates();
+    auto yyATMt = priceSurfEU->atmYoYSwapTimeRates();
+    auto yyATMd = priceSurfEU->atmYoYSwapDateRates();
 
     // Real dy = (Real)lag / 12.0;
     const Real crv[] = {0.024586, 0.0247575, 0.0249396, 0.0252596,
@@ -418,8 +413,7 @@ void InflationVolTest::testYoYPriceSurfaceToATM() {
 
 
 boost::unit_test_framework::test_suite* InflationVolTest::suite() {
-    boost::unit_test_framework::test_suite* suite
-        = BOOST_TEST_SUITE("yoyOptionletStripper (yoy inflation vol) tests");
+    auto* suite = BOOST_TEST_SUITE("yoyOptionletStripper (yoy inflation vol) tests");
 
     suite->add(QUANTLIB_TEST_CASE(&InflationVolTest::testYoYPriceSurfaceToATM));
     suite->add(QUANTLIB_TEST_CASE(&InflationVolTest::testYoYPriceSurfaceToVol));
