@@ -23,7 +23,6 @@
 #include "matrices.hpp"
 #include "utilities.hpp"
 #include <ql/experimental/math/moorepenroseinverse.hpp>
-#include <ql/math/initializers.hpp>
 #include <ql/math/matrix.hpp>
 #include <ql/math/matrixutilities/basisincompleteordered.hpp>
 #include <ql/math/matrixutilities/bicgstab.hpp>
@@ -305,7 +304,7 @@ void MatricesTest::testQRSolve() {
         Array b(A.rows());
 
         for (Size k=0; k < 10; ++k) {
-            for (double& iter : b) {
+            for (Real& iter : b) {
                 iter = rng.next().value;
             }
             const Array x = qrSolve(A, b, true);
@@ -330,7 +329,7 @@ void MatricesTest::testQRSolve() {
                     if (w[i] > threshold) {
                         const Real u = std::inner_product(U.column_begin(i),
                                                           U.column_end(i),
-                                                          b.begin(), 0.0)/w[i];
+                                                          b.begin(), Real(0.0))/w[i];
 
                         for (Size j=0; j<n; ++j) {
                             xr[j]  +=u*V[j][i];
@@ -403,7 +402,7 @@ void MatricesTest::testDeterminant() {
     MersenneTwisterUniformRng rng(1234);
     for (Size j=0; j<100; ++j) {
         Matrix m(3, 3, 0.0);
-        for (double& iter : m)
+        for (Real& iter : m)
             iter = rng.next().value;
 
         if ((j % 3) == 0U) {
@@ -593,7 +592,7 @@ void MatricesTest::testMoorePenroseInverse() {
 
     Real cached[6] = {1.153846153846152, 1.461538461538463, 1.384615384615384,
                       1.384615384615385, 1.461538461538462, 1.153846153846152};
-    QL_CONSTEXPR Real tol = 500.0 * QL_EPSILON;
+    constexpr double tol = 500.0 * QL_EPSILON;
 
     for (Size i = 0; i < 6; ++i) {
         if (std::abs(x[i] - cached[i]) > tol) {
@@ -606,7 +605,7 @@ void MatricesTest::testMoorePenroseInverse() {
     }
 
     Array y = A*x;
-    QL_CONSTEXPR Real tol2 = 2000.0 * QL_EPSILON;
+    constexpr double tol2 = 2000.0 * QL_EPSILON;
     for (Size i = 0; i < 6; ++i) {
         if (std::abs(y[i] - 260.0) > tol2) {
             BOOST_FAIL(
@@ -624,9 +623,8 @@ namespace matrices_test {
     class MatrixMult {
       public:
         explicit MatrixMult(Matrix m) : m_(std::move(m)) {}
-        Disposable<Array> operator()(const Array& x) const {
-            Array retVal = m_*x;
-            return retVal;
+        Array operator()(const Array& x) const {
+            return m_ * x;
         }
 
       private:
@@ -648,7 +646,7 @@ void MatricesTest::testIterativeSolvers() {
     Array b(3);
     b[0] = 1.0; b[1] = 0.5; b[2] = 3.0;
 
-    QL_CONSTEXPR Real relTol = 1e4*QL_EPSILON;
+    constexpr double relTol = 1e4 * QL_EPSILON;
 
     const Array x = BiCGstab(MatrixMult(M1), 3, relTol).solve(b).x;
     if (norm2(M1*x-b)/norm2(b) > relTol) {
@@ -664,7 +662,7 @@ void MatricesTest::testIterativeSolvers() {
                 << "\n  rel tolerance : " << relTol);
     }
     const Array errors = Array(u.errors.begin(), u.errors.end());
-    for (double error : errors) {
+    for (Real error : errors) {
         const Array x = GMRES(MatrixMult(M1), 10, 1.01 * error).solve(b, b).x;
 
         const Real calculated = norm2(M1*x-b)/norm2(b);
@@ -793,6 +791,73 @@ void MatricesTest::testSparseMatrixMemory() {
 
 }
 
+#define QL_CHECK_CLOSE_MATRIX(actual, expected)                             \
+    BOOST_REQUIRE(actual.rows() == expected.rows() &&                       \
+                  actual.columns() == expected.columns());                  \
+    for (auto i = 0u; i < actual.rows(); i++) {                             \
+        for (auto j = 0u; j < actual.columns(); j++) {                      \
+            QL_CHECK_CLOSE(actual(i, j), expected(i, j), 100 * QL_EPSILON); \
+        }                                                                   \
+    }                                                                       \
+
+void MatricesTest::testOperators() {
+
+    BOOST_TEST_MESSAGE("Testing matrix operators...");
+
+    auto get_matrix = []() {
+        return Matrix(2, 3, 4.0);
+    };
+
+    const auto m = get_matrix();
+
+    const auto negative = Matrix(2, 3, -4.0);
+    const auto lvalue_negative = -m;
+    const auto rvalue_negative = -get_matrix();
+
+    QL_CHECK_CLOSE_MATRIX(lvalue_negative, negative);
+    QL_CHECK_CLOSE_MATRIX(rvalue_negative, negative);
+
+    const auto matrix_sum = Matrix(2, 3, 8.0);
+    const auto lvalue_lvalue_sum = m + m;
+    const auto lvalue_rvalue_sum = m + get_matrix();
+    const auto rvalue_lvalue_sum = get_matrix() + m;
+    const auto rvalue_rvalue_sum = get_matrix() + get_matrix();
+
+    QL_CHECK_CLOSE_MATRIX(lvalue_lvalue_sum, matrix_sum);
+    QL_CHECK_CLOSE_MATRIX(lvalue_rvalue_sum, matrix_sum);
+    QL_CHECK_CLOSE_MATRIX(rvalue_lvalue_sum, matrix_sum);
+    QL_CHECK_CLOSE_MATRIX(rvalue_rvalue_sum, matrix_sum);
+
+    const auto matrix_difference = Matrix(2, 3, 0.0);
+    const auto lvalue_lvalue_difference = m - m;  // NOLINT(misc-redundant-expression)
+    const auto lvalue_rvalue_difference = m - get_matrix();
+    const auto rvalue_lvalue_difference = get_matrix() - m;
+    const auto rvalue_rvalue_difference = get_matrix() - get_matrix();
+
+    QL_CHECK_CLOSE_MATRIX(lvalue_lvalue_difference, matrix_difference);
+    QL_CHECK_CLOSE_MATRIX(lvalue_rvalue_difference, matrix_difference);
+    QL_CHECK_CLOSE_MATRIX(rvalue_lvalue_difference, matrix_difference);
+    QL_CHECK_CLOSE_MATRIX(rvalue_rvalue_difference, matrix_difference);
+
+    const auto scalar_product = Matrix(2, 3, 6.0);
+    const auto lvalue_real_product = m * 1.5;
+    const auto rvalue_real_product = get_matrix() * 1.5;
+    const auto real_lvalue_product = 1.5 * m;
+    const auto real_rvalue_product = 1.5 * get_matrix();
+
+    QL_CHECK_CLOSE_MATRIX(lvalue_real_product, scalar_product);
+    QL_CHECK_CLOSE_MATRIX(rvalue_real_product, scalar_product);
+    QL_CHECK_CLOSE_MATRIX(real_lvalue_product, scalar_product);
+    QL_CHECK_CLOSE_MATRIX(real_rvalue_product, scalar_product);
+
+    const auto scalar_quotient = Matrix(2, 3, 2.0);
+    const auto lvalue_real_quotient = m / 2.0;
+    const auto rvalue_real_quotient = get_matrix() / 2.0;
+
+    QL_CHECK_CLOSE_MATRIX(lvalue_real_quotient, scalar_quotient);
+    QL_CHECK_CLOSE_MATRIX(rvalue_real_quotient, scalar_quotient);
+}
+
 test_suite* MatricesTest::suite() {
     auto* suite = BOOST_TEST_SUITE("Matrix tests");
 
@@ -810,6 +875,7 @@ test_suite* MatricesTest::suite() {
     suite->add(QUANTLIB_TEST_CASE(&MatricesTest::testMoorePenroseInverse));
     suite->add(QUANTLIB_TEST_CASE(&MatricesTest::testIterativeSolvers));
     suite->add(QUANTLIB_TEST_CASE(&MatricesTest::testInitializers));
+    suite->add(QUANTLIB_TEST_CASE(&MatricesTest::testOperators));
     return suite;
 }
 
