@@ -41,16 +41,15 @@ class PiecewiseYieldCurveTest {
     static void testConvexMonotoneForwardConsistency();
     static void testLocalBootstrapConsistency();
 
+    static void testDefaultInstantiation();
+
     static void testObservability();
     static void testLiborFixing();
 
     static void testJpyLibor();
 
-    static void testDiscountCopy();
-    static void testForwardCopy();
-    static void testZeroCopy();
-
     static void testSwapRateHelperLastRelevantDate();
+    static void testSwapRateHelperSpotDate();
 
     static void testBadPreviousCurve();
 
@@ -59,6 +58,8 @@ class PiecewiseYieldCurveTest {
 
     static void testGlobalBootstrap();
 
+    static void testIterativeBootstrapRetries();
+
     static boost::unit_test_framework::test_suite* suite();
 };
 
