@@ -1,7 +1,7 @@
 /* -*- mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
 /*
- Copyright (C) 2015 Johannes Goettker-Schnetmann
+ Copyright (C) 2015 Johannes Göttker-Schnetmann
  Copyright (C) 2015, 2016 Klaus Spanderen
 
  This file is part of QuantLib, a free-software/open-source library
@@ -20,21 +20,9 @@
 
 #include "riskneutraldensitycalculator.hpp"
 #include "utilities.hpp"
-
-#include <ql/timegrid.hpp>
-#include <ql/time/calendars/nullcalendar.hpp>
-#include <ql/math/functional.hpp>
+#include <ql/instruments/vanillaoption.hpp>
 #include <ql/math/distributions/normaldistribution.hpp>
 #include <ql/math/integrals/gausslobattointegral.hpp>
-#include <ql/quotes/simplequote.hpp>
-#include <ql/processes/hestonprocess.hpp>
-#include <ql/processes/blackscholesprocess.hpp>
-#include <ql/instruments/vanillaoption.hpp>
-#include <ql/pricingengines/blackcalculator.hpp>
-#include <ql/pricingengines/vanilla/fdblackscholesvanillaengine.hpp>
-#include <ql/termstructures/volatility/equityfx/localconstantvol.hpp>
-#include <ql/termstructures/volatility/equityfx/hestonblackvolsurface.hpp>
-#include <ql/termstructures/volatility/equityfx/noexceptlocalvolsurface.hpp>
 #include <ql/methods/finitedifferences/utilities/bsmrndcalculator.hpp>
 #include <ql/methods/finitedifferences/utilities/cevrndcalculator.hpp>
 #include <ql/methods/finitedifferences/utilities/gbsmrndcalculator.hpp>
@@ -42,8 +30,18 @@
 #include <ql/methods/finitedifferences/utilities/localvolrndcalculator.hpp>
 #include <ql/methods/finitedifferences/utilities/squarerootprocessrndcalculator.hpp>
 #include <ql/models/equity/hestonmodel.hpp>
+#include <ql/pricingengines/blackcalculator.hpp>
+#include <ql/pricingengines/vanilla/fdblackscholesvanillaengine.hpp>
+#include <ql/processes/blackscholesprocess.hpp>
+#include <ql/processes/hestonprocess.hpp>
+#include <ql/quotes/simplequote.hpp>
+#include <ql/termstructures/volatility/equityfx/hestonblackvolsurface.hpp>
+#include <ql/termstructures/volatility/equityfx/localconstantvol.hpp>
+#include <ql/termstructures/volatility/equityfx/noexceptlocalvolsurface.hpp>
+#include <ql/time/calendars/nullcalendar.hpp>
+#include <ql/timegrid.hpp>
 #include <ql/types.hpp>
-#include <ql/functional.hpp>
+#include <utility>
 
 using namespace QuantLib;
 using namespace boost::unit_test_framework;
@@ -77,14 +75,12 @@ void RiskNeutralDensityCalculatorTest::testDensityAgainstOptionPrices() {
     const Time times[] = { 0.5, 1.0, 2.0 };
     const Real strikes[] = { 75.0, 100.0, 150.0 };
 
-    for (Size i=0; i < LENGTH(times); ++i) {
-        const Time t = times[i];
-        const Volatility stdDev = v*std::sqrt(t);
+    for (Real t : times) {
+        const Volatility stdDev = v * std::sqrt(t);
         const DiscountFactor df = rTS->discount(t);
         const Real fwd = s0*qTS->discount(t)/df;
 
-        for (Size j=0; j < LENGTH(strikes); ++j) {
-            const Real strike = strikes[j];
+        for (Real strike : strikes) {
             const Real xs = std::log(strike);
             const BlackCalculator blackCalc(
                 Option::Put, strike, fwd, stdDev, df);
@@ -163,11 +159,8 @@ void RiskNeutralDensityCalculatorTest::testBSMagainstHestonRND() {
     const Real strikes[] = { 7.5, 10, 15 };
     const Real probs[] = { 1e-6, 0.01, 0.5, 0.99, 1.0-1e-6 };
 
-    for (Size i=0; i < LENGTH(times); ++i) {
-        const Time t = times[i];
-
-        for (Size j=0; j < LENGTH(strikes); ++j) {
-            const Real strike = strikes[j];
+    for (Real t : times) {
+        for (Real strike : strikes) {
             const Real xs = std::log(strike);
 
             const Real expectedPDF = bsm.pdf(xs, t);
@@ -196,8 +189,7 @@ void RiskNeutralDensityCalculatorTest::testBSMagainstHestonRND() {
             }
         }
 
-        for (Size j=0; j < LENGTH(probs); ++j) {
-            const Real prob = probs[j];
+        for (Real prob : probs) {
             const Real expectedInvCDF = bsm.invcdf(prob, t);
             const Real calculatedInvCDF = heston.invcdf(prob, t);
 
@@ -220,22 +212,24 @@ namespace {
     // http://www.researchgate.net/publication/46493859_Implied_volatility_in_oil_markets
     class DumasParametricVolSurface : public BlackVolatilityTermStructure {
       public:
-        DumasParametricVolSurface(
-            Real b1, Real b2, Real b3, Real b4, Real b5,
-            const ext::shared_ptr<Quote>& spot,
-            const ext::shared_ptr<YieldTermStructure>& rTS,
-            const ext::shared_ptr<YieldTermStructure>& qTS)
-        : BlackVolatilityTermStructure(
-              0, NullCalendar(), Following, rTS->dayCounter()),
-          b1_(b1), b2_(b2), b3_(b3), b4_(b4), b5_(b5),
-          spot_(spot), rTS_(rTS), qTS_(qTS) {}
+        DumasParametricVolSurface(Real b1,
+                                  Real b2,
+                                  Real b3,
+                                  Real b4,
+                                  Real b5,
+                                  ext::shared_ptr<Quote> spot,
+                                  const ext::shared_ptr<YieldTermStructure>& rTS,
+                                  ext::shared_ptr<YieldTermStructure> qTS)
+        : BlackVolatilityTermStructure(0, NullCalendar(), Following, rTS->dayCounter()), b1_(b1),
+          b2_(b2), b3_(b3), b4_(b4), b5_(b5), spot_(std::move(spot)), rTS_(rTS),
+          qTS_(std::move(qTS)) {}
 
-        Date maxDate() const { return Date::maxDate(); }
-        Rate minStrike() const { return 0.0; }
-        Rate maxStrike() const { return QL_MAX_REAL; }
+        Date maxDate() const override { return Date::maxDate(); }
+        Rate minStrike() const override { return 0.0; }
+        Rate maxStrike() const override { return QL_MAX_REAL; }
 
       protected:
-        Volatility blackVolImpl(Time t, Real strike) const {
+        Volatility blackVolImpl(Time t, Real strike) const override {
             QL_REQUIRE(t >= 0.0, "t must be >= 0");
 
             if (t < QL_EPSILON)
@@ -256,11 +250,10 @@ namespace {
 
     class ProbWeightedPayoff {
       public:
-        ProbWeightedPayoff(
-            Time t,
-            const ext::shared_ptr<Payoff>& payoff,
-            const ext::shared_ptr<RiskNeutralDensityCalculator>& calc)
-      : t_(t), payoff_(payoff), calc_(calc) {}
+        ProbWeightedPayoff(Time t,
+                           ext::shared_ptr<Payoff> payoff,
+                           ext::shared_ptr<RiskNeutralDensityCalculator> calc)
+        : t_(t), payoff_(std::move(payoff)), calc_(std::move(calc)) {}
 
         Real operator()(Real x) const {
             return calc_->pdf(x, t_) * (*payoff_)(std::exp(x));
@@ -272,7 +265,7 @@ namespace {
         const ext::shared_ptr<RiskNeutralDensityCalculator> calc_;
     };
 
-    Disposable<std::vector<Time> > adaptiveTimeGrid(
+    std::vector<Time> adaptiveTimeGrid(
         Size maxStepsPerYear, Size minStepsPerYear, Real decay, Time endTime) {
         const Time maxDt = 1.0/maxStepsPerYear;
         const Time minDt = 1.0/minStepsPerYear;
@@ -421,33 +414,27 @@ void RiskNeutralDensityCalculatorTest::testLocalVolatilityRND() {
             spot, rTS, qTS, localVolSurface, dumasTimeGrid, 401, 0.1, 1e-8));
 
     const Real strikes[] = { 25, 50, 95, 100, 105, 150, 200, 400 };
-    const Date maturityDates[] = {
+    const std::vector<Date> maturities = {
         todaysDate + Period(1, Weeks),   todaysDate + Period(1, Months),
         todaysDate + Period(3, Months),  todaysDate + Period(6, Months),
         todaysDate + Period(12, Months), todaysDate + Period(18, Months),
         todaysDate + Period(2, Years),   todaysDate + Period(3, Years) };
-    const std::vector<Date> maturities(
-        maturityDates, maturityDates + LENGTH(maturityDates));
 
 
-    for (Size i=0; i < maturities.size(); ++i) {
+    for (auto maturity : maturities) {
         const Time expiry
-            = rTS->dayCounter().yearFraction(todaysDate, maturities[i]);
+            = rTS->dayCounter().yearFraction(todaysDate, maturity);
 
         const ext::shared_ptr<PricingEngine> engine(
             new FdBlackScholesVanillaEngine(
                 bsmProcess, std::max(Size(51), Size(expiry*101)),
                 201, 0, FdmSchemeDesc::Douglas(), true, b1));
 
-        const ext::shared_ptr<Exercise> exercise(
-            new EuropeanExercise(maturities[i]));
+        const ext::shared_ptr<Exercise> exercise(new EuropeanExercise(maturity));
 
-        for (Size k=0; k < LENGTH(strikes); ++k) {
-            const Real strike = strikes[k];
-            const ext::shared_ptr<StrikedTypePayoff> payoff(
-                new PlainVanillaPayoff(
-                    (strike > spot->value()) ? Option::Call : Option::Put
-                    , strike));
+        for (Real strike : strikes) {
+            const ext::shared_ptr<StrikedTypePayoff> payoff(new PlainVanillaPayoff(
+                (strike > spot->value()) ? Option::Call : Option::Put, strike));
 
             VanillaOption option(payoff, exercise);
             option.setPricingEngine(engine);
@@ -491,20 +478,19 @@ void RiskNeutralDensityCalculatorTest::testSquareRootProcessRND() {
             { 1.0, 0.6, 0.1, 0.75 },
             { 0.005, 0.6, 0.1, 0.05 } };
 
-    for (Size i=0; i < LENGTH(params); ++i) {
-        const SquareRootProcessRNDCalculator rndCalculator(
-            params[i].v0, params[i].kappa, params[i].theta, params[i].sigma);
+    for (const auto& param : params) {
+        const SquareRootProcessRNDCalculator rndCalculator(param.v0, param.kappa, param.theta,
+                                                           param.sigma);
 
         const Time t = 0.75;
-        const Time tInfty = 60.0/params[i].kappa;
+        const Time tInfty = 60.0 / param.kappa;
 
         const Real tol = 1e-10;
-        for (Real v = 1e-5; v < 1.0; v+=(v < params[i].theta) ? 0.005 : 0.1) {
+        for (Real v = 1e-5; v < 1.0; v += (v < param.theta) ? 0.005 : 0.1) {
 
             const Real cdfCalculated = rndCalculator.cdf(v, t);
             const Real cdfExpected = GaussLobattoIntegral(10000, 0.01*tol)(
-                ext::bind(&SquareRootProcessRNDCalculator::pdf,
-                          &rndCalculator, ext::placeholders::_1, t), 0, v);
+                [&](Real _x) { return rndCalculator.pdf(_x, t); }, 0, v);
 
             if (std::fabs(cdfCalculated - cdfExpected) > tol) {
                 BOOST_FAIL("failed to calculate cdf"
@@ -627,8 +613,7 @@ void RiskNeutralDensityCalculatorTest::testBlackScholesWithSkew() {
 
     const Real strikes[] = { 85, 75, 90, 110, 125, 150 };
 
-    for (Size i=0; i < LENGTH(strikes); ++i) {
-        const Real strike = strikes[i];
+    for (Real strike : strikes) {
         const Real logStrike = std::log(strike);
 
         const Real expected = hestonCalc.cdf(logStrike, maturity);
@@ -660,8 +645,7 @@ void RiskNeutralDensityCalculatorTest::testBlackScholesWithSkew() {
         }
     }
 
-    for (Size i=0; i < LENGTH(strikes); ++i) {
-        const Real strike = strikes[i];
+    for (Real strike : strikes) {
         const Real logStrike = std::log(strike);
 
         const Real expected = hestonCalc.pdf(logStrike, maturity)/strike;
@@ -695,9 +679,7 @@ void RiskNeutralDensityCalculatorTest::testBlackScholesWithSkew() {
     }
 
     const Real quantiles[] = { 0.05, 0.25, 0.5, 0.75, 0.95 };
-    for (Size i=0; i < LENGTH(quantiles); ++i) {
-        const Real quantile = quantiles[i];
-
+    for (Real quantile : quantiles) {
         const Real expected = std::exp(hestonCalc.invcdf(quantile, maturity));
         const Real calculatedGBSM = gbsmCalc.invcdf(quantile, maturity);
 
@@ -737,17 +719,17 @@ void RiskNeutralDensityCalculatorTest::testMassAtZeroCEVProcessRND() {
     const Time t = 2.75;
 
     const std::pair<Real, Real> params[] = {
-      std::make_pair(0.1, 1.6),
-      std::make_pair(0.01, 2.0),
-      std::make_pair(10.0, 0.35),
-      std::make_pair(50.0, 0.1)
+        {0.1, 1.6},
+        {0.01, 2.0},
+        {10.0, 0.35},
+        {50.0, 0.1}
     };
 
     const Real tol = 1e-4;
 
-    for (Size i=0; i < LENGTH(params); ++i) {
-        const Real alpha = params[i].first;
-        const Real beta = params[i].second;
+    for (const auto& param : params) {
+        const Real alpha = param.first;
+        const Real beta = param.second;
 
         const ext::shared_ptr<CEVRNDCalculator> calculator =
             ext::make_shared<CEVRNDCalculator>(f0, alpha, beta);
@@ -755,8 +737,7 @@ void RiskNeutralDensityCalculatorTest::testMassAtZeroCEVProcessRND() {
         const Real ax = 15.0*std::sqrt(t)*alpha*std::pow(f0, beta);
 
         const Real calculated = GaussLobattoIntegral(1000, 1e-8)(
-            ext::bind(&CEVRNDCalculator::pdf, calculator, ext::placeholders::_1, t),
-                      std::max(QL_EPSILON, f0-ax), f0+ax) +
+            [&](Real _x) -> Real { return calculator->pdf(_x, t); }, std::max(QL_EPSILON, f0-ax), f0+ax) +
             calculator->massAtZero(t);
 
         if (std::fabs(calculated - 1.0) > tol) {
@@ -805,7 +786,7 @@ void RiskNeutralDensityCalculatorTest::testCEVCDF() {
 }
 
 test_suite* RiskNeutralDensityCalculatorTest::experimental(SpeedLevel speed) {
-    test_suite* suite = BOOST_TEST_SUITE("Risk neutral density calculator tests");
+    auto* suite = BOOST_TEST_SUITE("Risk neutral density calculator tests");
 
     suite->add(QUANTLIB_TEST_CASE(
         &RiskNeutralDensityCalculatorTest::testDensityAgainstOptionPrices));
