@@ -42,20 +42,13 @@
 #include <ql/termstructures/volatility/equityfx/hestonblackvolsurface.hpp>
 #include <ql/termstructures/volatility/equityfx/noexceptlocalvolsurface.hpp>
 #include <ql/experimental/models/squarerootclvmodel.hpp>
-#include <ql/experimental/models/hestonslvfdmmodel.hpp>
-#include <ql/experimental/processes/hestonslvprocess.hpp>
-#include <ql/experimental/finitedifferences/fdhestondoublebarrierengine.hpp>
-#include <ql/experimental/barrieroption/analyticdoublebarrierbinaryengine.hpp>
+#include <ql/models/equity/hestonslvfdmmodel.hpp>
+#include <ql/processes/hestonslvprocess.hpp>
+#include <ql/pricingengines/barrier/fdhestondoublebarrierengine.hpp>
+#include <ql/pricingengines/barrier/analyticdoublebarrierbinaryengine.hpp>
 #include <ql/experimental/volatility/sabrvoltermstructure.hpp>
 
-#if defined(__GNUC__) && !defined(__clang__) && BOOST_VERSION > 106300
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
-#endif
 #include <boost/math/distributions/non_central_chi_squared.hpp>
-#if defined(__GNUC__) && !defined(__clang__) && BOOST_VERSION > 106300
-#pragma GCC diagnostic pop
-#endif
 
 #include <set>
 #include <utility>
@@ -92,7 +85,7 @@ void SquareRootCLVModelTest::testSquareRootCLVVanillaPricing() {
     const Date todaysDate(5, Oct, 2016);
     Settings::instance().evaluationDate() = todaysDate;
 
-    const DayCounter dc = ActualActual();
+    const DayCounter dc = ActualActual(ActualActual::ISDA);
     const Date maturityDate = todaysDate + Period(3, Months);
     const Time maturity = dc.yearFraction(todaysDate, maturityDate);
 
@@ -137,7 +130,7 @@ void SquareRootCLVModelTest::testSquareRootCLVVanillaPricing() {
     const chi_squared_type dist(df, ncp);
         
     const Real strikes[] = { 50, 75, 100, 125, 150, 200 };
-    for (double strike : strikes) {
+    for (Real strike : strikes) {
         const Option::Type optionType = (strike > fwd) ? Option::Call : Option::Put;
 
         const Real expected = BlackCalculator(
@@ -147,7 +140,7 @@ void SquareRootCLVModelTest::testSquareRootCLVVanillaPricing() {
 
         const CLVModelPayoff clvModelPayoff(optionType, strike, g);
 
-        const ext::function<Real(Real)> f = [&](Real xi) {
+        const ext::function<Real(Real)> f = [&](Real xi) -> Real {
             return clvModelPayoff(xi) * boost::math::pdf(dist, xi);
         };
 
@@ -235,7 +228,7 @@ void SquareRootCLVModelTest::testSquareRootCLVMappingFunction() {
 
         const Real fwd = s0*qTS->discount(m)/rTS->discount(m);
 
-        for (double strike : strikes) {
+        for (Real strike : strikes) {
             const Option::Type optionType = (strike > fwd) ? Option::Call : Option::Put;
 
             const Real expected = BlackCalculator(
@@ -245,7 +238,7 @@ void SquareRootCLVModelTest::testSquareRootCLVMappingFunction() {
 
             const CLVModelPayoff clvModelPayoff(optionType, strike, [&](Real x) { return g(t, x); });
 
-            const ext::function<Real(Real)> f = [&](Real xi) {
+            const ext::function<Real(Real)> f = [&](Real xi) -> Real {
                 return clvModelPayoff(xi) * boost::math::pdf(dist, xi);
             };
 
@@ -288,13 +281,13 @@ namespace square_root_clv_model {
             const Array diff = values(params);
 
             Real retVal = 0.0;
-            for (double i : diff)
+            for (Real i : diff)
                 retVal += i * i;
 
             return retVal;
         }
 
-        Disposable<Array> values(const Array& params) const override {
+        Array values(const Array& params) const override {
             const Real theta = params[0];
             const Real kappa = params[1];
             const Real sigma = params[2];
@@ -371,8 +364,8 @@ namespace square_root_clv_model {
                         const Real strike = strikes_[k];
 
                         const Real payoff = (strike < 1.0)
-                            ?  s1 * std::max(0.0, strike - s2/s1)
-                            :  s1 * std::max(0.0, s2/s1 - strike);
+                            ?  Real(s1 * std::max(0.0, strike - s2/s1))
+                            :  Real(s1 * std::max(0.0, s2/s1 - strike));
 
                         stats[k].add(payoff);
                     }
@@ -623,8 +616,8 @@ void SquareRootCLVModelTest::testForwardSkew() {
             for (Size j=0; j < LENGTH(strikes); ++j) {
                 const Real strike = strikes[j];
                     slvStats[i][j].add((strike < 1.0)
-                        ? S_t1 * std::max(0.0, strike - S_T1/S_t1)
-                        : S_t1 * std::max(0.0, S_T1/S_t1 - strike));
+                        ? Real(S_t1 * std::max(0.0, strike - S_T1/S_t1))
+                        : Real(S_t1 * std::max(0.0, S_T1/S_t1 - strike)));
             }
 
         }
