@@ -141,7 +141,7 @@ void SwaptionTest::testStrikeDependency() {
                 std::vector<Real> values;
                 std::vector<Real> values_cash;
                 Volatility vol = 0.20;
-                for (double strike : strikes) {
+                for (Real strike : strikes) {
                     ext::shared_ptr<VanillaSwap> swap =
                         MakeVanillaSwap(length, vars.index, strike)
                             .withEffectiveDate(startDate)
@@ -151,7 +151,6 @@ void SwaptionTest::testStrikeDependency() {
                             .withType(k);
                     ext::shared_ptr<Swaption> swaption =
                         vars.makeSwaption(swap,exerciseDate,vol);
-                    // FLOATING_POINT_EXCEPTION
                     values.push_back(swaption->NPV());
                     ext::shared_ptr<Swaption> swaption_cash =
                         vars.makeSwaption(swap,exerciseDate,vol,
@@ -160,7 +159,7 @@ void SwaptionTest::testStrikeDependency() {
                 }
                 // and check that they go the right way
                 if (k == Swap::Payer) {
-                    auto it = std::adjacent_find(values.begin(), values.end(), std::less<Real>());
+                    auto it = std::adjacent_find(values.begin(), values.end(), std::less<>());
                     if (it != values.end()) {
                         Size n = it - values.begin();
                         BOOST_ERROR("NPV of Payer swaption with delivery settlement"
@@ -172,8 +171,7 @@ void SwaptionTest::testStrikeDependency() {
                                     << "\nvalue:        " << values[n + 1]
                                     << " at strike: " << io::rate(strikes[n + 1]));
                     }
-                    auto it_cash = std::adjacent_find(values_cash.begin(), values_cash.end(),
-                                                      std::less<Real>());
+                    auto it_cash = std::adjacent_find(values_cash.begin(), values_cash.end(), std::less<>());
                     if (it_cash != values_cash.end()) {
                         Size n = it_cash - values_cash.begin();
                         BOOST_ERROR("NPV of Payer swaption with cash settlement"
@@ -187,7 +185,7 @@ void SwaptionTest::testStrikeDependency() {
                     }
                 } else {
                     auto it =
-                        std::adjacent_find(values.begin(), values.end(), std::greater<Real>());
+                        std::adjacent_find(values.begin(), values.end(), std::greater<>());
                     if (it != values.end()) {
                         Size n = it - values.begin();
                         BOOST_ERROR("NPV of Receiver swaption with delivery settlement"
@@ -199,8 +197,7 @@ void SwaptionTest::testStrikeDependency() {
                                     << "\nvalue:        " << values[n + 1]
                                     << " at strike: " << io::rate(strikes[n + 1]));
                     }
-                    auto it_cash = std::adjacent_find(values_cash.begin(), values_cash.end(),
-                                                      std::greater<Real>());
+                    auto it_cash = std::adjacent_find(values_cash.begin(), values_cash.end(), std::greater<>());
                     if (it_cash != values_cash.end()) {
                         Size n = it_cash - values_cash.begin();
                         BOOST_ERROR("NPV of Receiver swaption with cash settlement"
@@ -238,7 +235,7 @@ void SwaptionTest::testSpreadDependency() {
                 // store the results for different rates...
                 std::vector<Real> values;
                 std::vector<Real> values_cash;
-                for (double spread : spreads) {
+                for (Real spread : spreads) {
                     ext::shared_ptr<VanillaSwap> swap =
                         MakeVanillaSwap(length, vars.index, 0.06)
                             .withFixedLegTenor(1 * Years)
@@ -248,7 +245,6 @@ void SwaptionTest::testSpreadDependency() {
                             .withType(k);
                     ext::shared_ptr<Swaption> swaption =
                         vars.makeSwaption(swap,exerciseDate,0.20);
-                    // FLOATING_POINT_EXCEPTION
                     values.push_back(swaption->NPV());
                     ext::shared_ptr<Swaption> swaption_cash =
                         vars.makeSwaption(swap,exerciseDate,0.20,
@@ -258,7 +254,7 @@ void SwaptionTest::testSpreadDependency() {
                 // and check that they go the right way
                 if (k == Swap::Payer) {
                     auto it =
-                        std::adjacent_find(values.begin(), values.end(), std::greater<Real>());
+                        std::adjacent_find(values.begin(), values.end(), std::greater<>());
                     if (it != values.end()) {
                         Size n = it - values.begin();
                         BOOST_ERROR("NPV is decreasing with the spread "
@@ -268,8 +264,7 @@ void SwaptionTest::testSpreadDependency() {
                                     << io::rate(spreads[n]) << "\nvalue:         " << values[n + 1]
                                     << " for spread: " << io::rate(spreads[n + 1]));
                     }
-                    auto it_cash = std::adjacent_find(values_cash.begin(), values_cash.end(),
-                                                      std::greater<Real>());
+                    auto it_cash = std::adjacent_find(values_cash.begin(), values_cash.end(), std::greater<>());
                     if (it_cash != values_cash.end()) {
                         Size n = it_cash - values_cash.begin();
                         BOOST_ERROR("NPV is decreasing with the spread "
@@ -280,7 +275,7 @@ void SwaptionTest::testSpreadDependency() {
                                     << " for spread: " << io::rate(spreads[n + 1]));
                     }
                 } else {
-                    auto it = std::adjacent_find(values.begin(), values.end(), std::less<Real>());
+                    auto it = std::adjacent_find(values.begin(), values.end(), std::less<>());
                     if (it != values.end()) {
                         Size n = it - values.begin();
                         BOOST_ERROR("NPV is increasing with the spread "
@@ -291,8 +286,7 @@ void SwaptionTest::testSpreadDependency() {
                                     << "\nvalue:  " << values[n + 1]
                                     << " for spread: " << io::rate(spreads[n + 1]));
                     }
-                    auto it_cash = std::adjacent_find(values_cash.begin(), values_cash.end(),
-                                                      std::less<Real>());
+                    auto it_cash = std::adjacent_find(values_cash.begin(), values_cash.end(), std::less<>());
                     if (it_cash != values_cash.end()) {
                         Size n = it_cash - values_cash.begin();
                         BOOST_ERROR("NPV is increasing with the spread "
@@ -326,7 +320,7 @@ void SwaptionTest::testSpreadTreatment() {
                 Date startDate =
                     vars.calendar.advance(exerciseDate,
                                           vars.settlementDays,Days);
-                for (double spread : spreads) {
+                for (Real spread : spreads) {
                     ext::shared_ptr<VanillaSwap> swap =
                         MakeVanillaSwap(length, vars.index, 0.06)
                             .withFixedLegTenor(1 * Years)
@@ -334,7 +328,6 @@ void SwaptionTest::testSpreadTreatment() {
                             .withEffectiveDate(startDate)
                             .withFloatingLegSpread(spread)
                             .withType(k);
-                    // FLOATING_POINT_EXCEPTION
                     Spread correction = spread * swap->floatingLegBPS() / swap->fixedLegBPS();
                     ext::shared_ptr<VanillaSwap> equivalentSwap =
                         MakeVanillaSwap(length, vars.index, 0.06 + correction)
@@ -379,6 +372,8 @@ void SwaptionTest::testCachedValue() {
 
     using namespace swaption_test;
 
+    bool usingAtParCoupons = IborCoupon::Settings::instance().usingAtParCoupons();
+
     CommonVars vars;
 
     vars.today = Date(13, March, 2002);
@@ -397,13 +392,8 @@ void SwaptionTest::testCachedValue() {
     ext::shared_ptr<Swaption> swaption =
         vars.makeSwaption(swap, exerciseDate, 0.20);
 
-    Real cachedNPV;
-    if (IborCoupon::usingAtParCoupons())
-        cachedNPV = 0.036418158579;
-    else
-        cachedNPV = 0.036421429684;
+    Real cachedNPV = usingAtParCoupons ? 0.036418158579 : 0.036421429684;
 
-    // FLOATING_POINT_EXCEPTION
     if (std::fabs(swaption->NPV()-cachedNPV) > 1.0e-12)
         BOOST_ERROR("failed to reproduce cached swaption value:\n" <<
                     std::fixed << std::setprecision(12) <<
@@ -429,7 +419,7 @@ void SwaptionTest::testVega() {
         Date startDate = vars.calendar.advance(exerciseDate,
                                            vars.settlementDays*Days);
         for (auto& length : lengths) {
-            for (double strike : strikes) {
+            for (Real strike : strikes) {
                 for (Size h=0; h<LENGTH(type); h++) {
                     ext::shared_ptr<VanillaSwap> swap =
                         MakeVanillaSwap(length, vars.index, strike)
@@ -438,10 +428,9 @@ void SwaptionTest::testVega() {
                             .withFixedLegDayCount(vars.fixedDayCount)
                             .withFloatingLegSpread(0.0)
                             .withType(type[h]);
-                    for (double vol : vols) {
+                    for (Real vol : vols) {
                         ext::shared_ptr<Swaption> swaption =
                             vars.makeSwaption(swap, exerciseDate, vol, types[h], methods[h]);
-                        // FLOATING_POINT_EXCEPTION
                         ext::shared_ptr<Swaption> swaption1 = vars.makeSwaption(
                             swap, exerciseDate, vol - shift, types[h], methods[h]);
                         ext::shared_ptr<Swaption> swaption2 = vars.makeSwaption(
@@ -555,7 +544,6 @@ void SwaptionTest::testCashSettledSwaptions() {
             const Leg& swapFixedLeg_a365 = swap_a365->fixedLeg();
 
             // FlatForward curves
-            // FLOATING_POINT_EXCEPTION
             Handle<YieldTermStructure> termStructure_u360(
                 ext::shared_ptr<YieldTermStructure>(
                     new FlatForward(vars.settlement,swap_u360->fairRate(),
@@ -807,7 +795,7 @@ void SwaptionTest::testImpliedVolatility() {
             Date startDate = vars.calendar.advance(exerciseDate,
                                                    vars.settlementDays, Days);
 
-            for (double& strike : strikes) {
+            for (Real& strike : strikes) {
                 for (auto& k : type) {
                     ext::shared_ptr<VanillaSwap> swap =
                         MakeVanillaSwap(length, vars.index, strike)
@@ -817,7 +805,7 @@ void SwaptionTest::testImpliedVolatility() {
                             .withFloatingLegSpread(0.0)
                             .withType(k);
                     for (Size h=0; h<LENGTH(types); h++) {
-                        for (double vol : vols) {
+                        for (Real vol : vols) {
                             ext::shared_ptr<Swaption> swaption =
                                 vars.makeSwaption(swap, exerciseDate, vol, types[h], methods[h],
                                                   BlackSwaptionEngine::DiscountCurve);
@@ -922,10 +910,10 @@ void checkSwaptionDelta(bool useBachelierVol)
     Rate strikes[] = { 0.03, 0.04, 0.05, 0.06, 0.07 };
     Volatility vols[] = { 0.0, 0.10, 0.20, 0.30, 0.70, 0.90 };
 
-    for (double vol : vols) {
+    for (Real vol : vols) {
         for (auto exercise : exercises) {
             for (auto& length : lengths) {
-                for (double& strike : strikes) {
+                for (Real& strike : strikes) {
                     for (Size h=0; h<LENGTH(type); h++) {
                         Volatility volatility = useBachelierVol ? vol / 100.0 : vol;
                         ext::shared_ptr<Engine> swaptionEngine = makeConstVolEngine<Engine>(
