@@ -24,8 +24,8 @@
 #include "utilities.hpp"
 #include <ql/indexes/swap/euriborswap.hpp>
 #include <ql/quotes/simplequote.hpp>
-#include <ql/termstructures/volatility/swaption/swaptionvolcube2.hpp>
-#include <ql/termstructures/volatility/swaption/swaptionvolcube1.hpp>
+#include <ql/termstructures/volatility/swaption/interpolatedswaptionvolatilitycube.hpp>
+#include <ql/termstructures/volatility/swaption/sabrswaptionvolatilitycube.hpp>
 #include <ql/termstructures/volatility/swaption/spreadedswaptionvol.hpp>
 #include <ql/termstructures/volatility/sabrsmilesection.hpp>
 #include <ql/utilities/dataformatters.hpp>
@@ -167,7 +167,7 @@ void SwaptionVolatilityCubeTest::testSabrNormalVolatility() {
     }
     std::vector<bool> isParameterFixed(4, false);
 
-    SwaptionVolCube1 volCube(vars.normalVolMatrix, vars.cube.tenors.options, vars.cube.tenors.swaps,
+    SabrSwaptionVolatilityCube volCube(vars.normalVolMatrix, vars.cube.tenors.options, vars.cube.tenors.swaps,
                              vars.cube.strikeSpreads, vars.cube.volSpreadsHandle,
                              vars.swapIndexBase, vars.shortSwapIndexBase, vars.vegaWeighedSmileFit,
                              parametersGuess, isParameterFixed, true);
@@ -183,7 +183,7 @@ void SwaptionVolatilityCubeTest::testAtmVols() {
 
     CommonVars vars;
 
-    SwaptionVolCube2 volCube(vars.atmVolMatrix,
+    InterpolatedSwaptionVolatilityCube volCube(vars.atmVolMatrix,
                              vars.cube.tenors.options,
                              vars.cube.tenors.swaps,
                              vars.cube.strikeSpreads,
@@ -204,7 +204,7 @@ void SwaptionVolatilityCubeTest::testSmile() {
 
     CommonVars vars;
 
-    SwaptionVolCube2 volCube(vars.atmVolMatrix,
+    InterpolatedSwaptionVolatilityCube volCube(vars.atmVolMatrix,
                              vars.cube.tenors.options,
                              vars.cube.tenors.swaps,
                              vars.cube.strikeSpreads,
@@ -240,7 +240,7 @@ void SwaptionVolatilityCubeTest::testSabrVols() {
     }
     std::vector<bool> isParameterFixed(4, false);
 
-    SwaptionVolCube1 volCube(vars.atmVolMatrix,
+    SabrSwaptionVolatilityCube volCube(vars.atmVolMatrix,
                              vars.cube.tenors.options,
                              vars.cube.tenors.swaps,
                              vars.cube.strikeSpreads,
@@ -282,7 +282,7 @@ void SwaptionVolatilityCubeTest::testSpreadedCube() {
     std::vector<bool> isParameterFixed(4, false);
 
     Handle<SwaptionVolatilityStructure> volCube( ext::shared_ptr<SwaptionVolatilityStructure>(new
-        SwaptionVolCube1(vars.atmVolMatrix,
+        SabrSwaptionVolatilityCube(vars.atmVolMatrix,
                          vars.cube.tenors.options,
                          vars.cube.tenors.swaps,
                          vars.cube.strikeSpreads,
@@ -371,9 +371,9 @@ void SwaptionVolatilityCubeTest::testObservability() {
     std::vector<bool> isParameterFixed(4, false);
 
     std::string description;
-    ext::shared_ptr<SwaptionVolCube1> volCube1_0, volCube1_1;
+    ext::shared_ptr<SabrSwaptionVolatilityCube> volCube1_0, volCube1_1;
     // VolCube created before change of reference date
-    volCube1_0 = ext::shared_ptr<SwaptionVolCube1>(new SwaptionVolCube1(vars.atmVolMatrix,
+    volCube1_0 = ext::shared_ptr<SabrSwaptionVolatilityCube>(new SabrSwaptionVolatilityCube(vars.atmVolMatrix,
                                                                 vars.cube.tenors.options,
                                                                 vars.cube.tenors.swaps,
                                                                 vars.cube.strikeSpreads,
@@ -391,7 +391,7 @@ void SwaptionVolatilityCubeTest::testObservability() {
                                           vars.conventions.optionBdc);
 
     // VolCube created after change of reference date
-    volCube1_1 = ext::shared_ptr<SwaptionVolCube1>(new SwaptionVolCube1(vars.atmVolMatrix,
+    volCube1_1 = ext::shared_ptr<SabrSwaptionVolatilityCube>(new SabrSwaptionVolatilityCube(vars.atmVolMatrix,
                                                                 vars.cube.tenors.options,
                                                                 vars.cube.tenors.swaps,
                                                                 vars.cube.strikeSpreads,
@@ -429,9 +429,9 @@ void SwaptionVolatilityCubeTest::testObservability() {
 
     Settings::instance().evaluationDate() = referenceDate;
 
-    ext::shared_ptr<SwaptionVolCube2> volCube2_0, volCube2_1;
+    ext::shared_ptr<InterpolatedSwaptionVolatilityCube> volCube2_0, volCube2_1;
     // VolCube created before change of reference date
-    volCube2_0 = ext::make_shared<SwaptionVolCube2>(vars.atmVolMatrix,
+    volCube2_0 = ext::make_shared<InterpolatedSwaptionVolatilityCube>(vars.atmVolMatrix,
                                                                 vars.cube.tenors.options,
                                                                 vars.cube.tenors.swaps,
                                                                 vars.cube.strikeSpreads,
@@ -444,7 +444,7 @@ void SwaptionVolatilityCubeTest::testObservability() {
                                           vars.conventions.optionBdc);
 
     // VolCube created after change of reference date
-    volCube2_1 = ext::make_shared<SwaptionVolCube2>(vars.atmVolMatrix,
+    volCube2_1 = ext::make_shared<InterpolatedSwaptionVolatilityCube>(vars.atmVolMatrix,
                                                                 vars.cube.tenors.options,
                                                                 vars.cube.tenors.swaps,
                                                                 vars.cube.strikeSpreads,
@@ -502,7 +502,7 @@ void SwaptionVolatilityCubeTest::testSabrParameters() {
     }
     std::vector<bool> isParameterFixed(4, false);
 
-    SwaptionVolCube1 volCube(vars.atmVolMatrix,
+    SabrSwaptionVolatilityCube volCube(vars.atmVolMatrix,
                              vars.cube.tenors.options,
                              vars.cube.tenors.swaps,
                              vars.cube.strikeSpreads,
